
KAA_DIR = '~/.kaa/'
KAA_SCRIPT = '__kaa__.py'
LOGDIR = 'log'
HISTDIR = 'history'
HIST_DBNAME = 'hist.db'
SPELLCHECKER_PWL = 'spell_pwl'

NEWLINES = ['auto', 'dos', 'unix']
NEWLINE_CHARS = {
    'auto': None,
    'dos': '\r\n',
    'unix': '\n'
}
