/* Generated by Cython 0.29.33 */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_33"
#define CYTHON_HEX_VERSION 0x001D21F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PY_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE___gappedbuf
#define __PYX_HAVE_API___gappedbuf
/* Early includes */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "pythread.h"
#include "_gappedbufre.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "_gappedbuf/_gappedbuf.pyx",
  "stringsource",
  "_gappedbuf/_gappedbuf.pxd",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
};

/*--- Type declarations ---*/
struct GappedBufferObj;

/* "_gappedbuf.pxd":2
 * 
 * cdef public class GappedBuffer [object GappedBufferObj, type GappedBufferType]:             # <<<<<<<<<<<<<<
 *     cdef public Py_ssize_t bufsize
 *     cdef public Py_ssize_t numelems
 */
struct GappedBufferObj {
  PyObject_HEAD
  struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *__pyx_vtab;
  Py_ssize_t bufsize;
  Py_ssize_t numelems;
  Py_ssize_t gap;
  Py_ssize_t gapsize;
  Py_UCS4 *buf;
};

__PYX_EXTERN_C DL_EXPORT(PyTypeObject) GappedBufferType;

__PYX_EXTERN_C DL_EXPORT(PyTypeObject) GappedBufferType;

/* "_gappedbuf.pyx":18
 *     sys.modules['_gappedbufre'] = PyInit__gappedbufre()
 * 
 * cdef class GappedBuffer:             # <<<<<<<<<<<<<<
 *     def __cinit__(self):
 *         DEF BUFSIZE = 4096
 */

struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer {
  void (*_resize_buf)(struct GappedBufferObj *, Py_ssize_t);
  void (*_expand_gap)(struct GappedBufferObj *, Py_ssize_t);
  void (*_shrink_gap)(struct GappedBufferObj *);
  void (*_move_gap)(struct GappedBufferObj *, Py_ssize_t);
  Py_UCS4 (*_get_max)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t);
  void (*_copy_chars1)(struct GappedBufferObj *, void *, Py_ssize_t, Py_ssize_t);
  void (*_copy_chars2)(struct GappedBufferObj *, void *, Py_ssize_t, Py_ssize_t);
  void (*_copy_chars4)(struct GappedBufferObj *, void *, Py_ssize_t, Py_ssize_t);
  int (*_get_slice)(struct GappedBufferObj *, PyObject *, Py_ssize_t *, Py_ssize_t *);
  PyObject *(*_insert)(struct GappedBufferObj *, Py_ssize_t, PyObject *);
  PyObject *(*insert)(struct GappedBufferObj *, Py_ssize_t, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*append)(struct GappedBufferObj *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_delete)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t);
  PyObject *(*delete)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch);
  PyObject *(*replace)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch);
  PyObject *(*getint)(struct GappedBufferObj *, Py_ssize_t, int __pyx_skip_dispatch);
  PyObject *(*getints)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch);
  PyObject *(*_insertints)(struct GappedBufferObj *, Py_ssize_t, PyObject *);
  PyObject *(*insertints)(struct GappedBufferObj *, Py_ssize_t, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*appendints)(struct GappedBufferObj *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*replaceints)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*setints)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, unsigned long, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *__pyx_vtabptr_10_gappedbuf_GappedBuffer;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned long __Pyx_PyInt_As_unsigned_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_long(unsigned long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* ObjectAsUCS4.proto */
#define __Pyx_PyObject_AsPy_UCS4(x)\
    (likely(PyUnicode_Check(x)) ? __Pyx_PyUnicode_AsPy_UCS4(x) : __Pyx__PyObject_AsPy_UCS4(x))
static Py_UCS4 __Pyx__PyObject_AsPy_UCS4(PyObject*);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_10_gappedbuf_12GappedBuffer__resize_buf(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_size); /* proto*/
static void __pyx_f_10_gappedbuf_12GappedBuffer__expand_gap(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_size); /* proto*/
static void __pyx_f_10_gappedbuf_12GappedBuffer__shrink_gap(struct GappedBufferObj *__pyx_v_self); /* proto*/
static void __pyx_f_10_gappedbuf_12GappedBuffer__move_gap(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer__insert(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_insert(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_append(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer__delete(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_delete(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_replace(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s, int __pyx_skip_dispatch); /* proto*/
static Py_UCS4 __pyx_f_10_gappedbuf_12GappedBuffer__get_max(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto*/
static void __pyx_f_10_gappedbuf_12GappedBuffer__copy_chars1(struct GappedBufferObj *__pyx_v_self, void *__pyx_v_dest, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto*/
static void __pyx_f_10_gappedbuf_12GappedBuffer__copy_chars2(struct GappedBufferObj *__pyx_v_self, void *__pyx_v_dest, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto*/
static void __pyx_f_10_gappedbuf_12GappedBuffer__copy_chars4(struct GappedBufferObj *__pyx_v_self, void *__pyx_v_dest, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_get(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_10_gappedbuf_12GappedBuffer__get_slice(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_obj, Py_ssize_t *__pyx_v_f, Py_ssize_t *__pyx_v_t); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_getint(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_pos, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_getints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer__insertints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_insertints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_appendints(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_replaceints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_setints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, unsigned long __pyx_v_v, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'rtdef' */

/* Module declarations from '_gappedbuf' */
static PyTypeObject *__pyx_ptype_10_gappedbuf_GappedBuffer = 0;
#define __Pyx_MODULE_NAME "_gappedbuf"
extern int __pyx_module_is_main__gappedbuf;
int __pyx_module_is_main__gappedbuf = 0;

/* Implementation of '_gappedbuf' */
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_TypeError;
static const char __pyx_k_c[] = "c";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_v[] = "v";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_nums[] = "nums";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_begin[] = "begin";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_delete[] = "delete";
static const char __pyx_k_getint[] = "getint";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_insert[] = "insert";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_comp_ne[] = "comp_ne";
static const char __pyx_k_getints[] = "getints";
static const char __pyx_k_modules[] = "modules";
static const char __pyx_k_replace[] = "replace";
static const char __pyx_k_setints[] = "setints";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_gappedbuf[] = "_gappedbuf";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_appendints[] = "appendints";
static const char __pyx_k_insertints[] = "insertints";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_gappedbufre[] = "_gappedbufre";
static const char __pyx_k_replaceints[] = "replaceints";
static const char __pyx_k_GappedBuffer[] = "GappedBuffer";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_init_gappedbufre[] = "init_gappedbufre";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_Invalid_range_d_d_d[] = "Invalid range: %d, %d, %d";
static const char __pyx_k_step_is_not_supported[] = "step is not supported";
static const char __pyx_k_Invalid_index_value_d_d[] = "Invalid index value: %d, %d";
static const char __pyx_k_Invalid_string_length_d[] = "Invalid string length: %d";
static const char __pyx_k_gappedbuf__gappedbuf_pyx[] = "_gappedbuf/_gappedbuf.pyx";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static PyObject *__pyx_n_s_GappedBuffer;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_kp_s_Invalid_index_value_d_d;
static PyObject *__pyx_kp_s_Invalid_range_d_d_d;
static PyObject *__pyx_kp_s_Invalid_string_length_d;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_appendints;
static PyObject *__pyx_n_s_begin;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_comp_ne;
static PyObject *__pyx_n_s_delete;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_gappedbuf;
static PyObject *__pyx_kp_s_gappedbuf__gappedbuf_pyx;
static PyObject *__pyx_n_s_gappedbufre;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getint;
static PyObject *__pyx_n_s_getints;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_init_gappedbufre;
static PyObject *__pyx_n_s_insert;
static PyObject *__pyx_n_s_insertints;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_modules;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_nums;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_replace;
static PyObject *__pyx_n_s_replaceints;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_setints;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_kp_s_step_is_not_supported;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_v;
static PyObject *__pyx_pf_10_gappedbuf_init_gappedbufre(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static int __pyx_pf_10_gappedbuf_12GappedBuffer___cinit__(struct GappedBufferObj *__pyx_v_self); /* proto */
static void __pyx_pf_10_gappedbuf_12GappedBuffer_2__dealloc__(struct GappedBufferObj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_4insert(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_6append(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_8delete(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_10replace(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_12get(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto */
static Py_ssize_t __pyx_pf_10_gappedbuf_12GappedBuffer_14__len__(struct GappedBufferObj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_16__getitem__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static int __pyx_pf_10_gappedbuf_12GappedBuffer_18__setitem__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_10_gappedbuf_12GappedBuffer_20__delitem__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_22findchr(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_c, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_24rfindchr(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_c, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_26getint(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_pos); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_28getints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_30insertints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_32appendints(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_34replaceints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_36setints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, unsigned long __pyx_v_v); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_38findint(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_nums, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_v_comp_ne); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_40rfindint(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_nums, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_v_comp_ne); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_7bufsize___get__(struct GappedBufferObj *__pyx_v_self); /* proto */
static int __pyx_pf_10_gappedbuf_12GappedBuffer_7bufsize_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_8numelems___get__(struct GappedBufferObj *__pyx_v_self); /* proto */
static int __pyx_pf_10_gappedbuf_12GappedBuffer_8numelems_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_3gap___get__(struct GappedBufferObj *__pyx_v_self); /* proto */
static int __pyx_pf_10_gappedbuf_12GappedBuffer_3gap_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_7gapsize___get__(struct GappedBufferObj *__pyx_v_self); /* proto */
static int __pyx_pf_10_gappedbuf_12GappedBuffer_7gapsize_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_42__reduce_cython__(CYTHON_UNUSED struct GappedBufferObj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_44__setstate_cython__(CYTHON_UNUSED struct GappedBufferObj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_10_gappedbuf_GappedBuffer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_codeobj__5;
/* Late includes */

/* "_gappedbuf.pyx":14
 * 
 * 
 * def init_gappedbufre():             # <<<<<<<<<<<<<<
 *     import sys
 *     sys.modules['_gappedbufre'] = PyInit__gappedbufre()
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_1init_gappedbufre(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10_gappedbuf_1init_gappedbufre = {"init_gappedbufre", (PyCFunction)__pyx_pw_10_gappedbuf_1init_gappedbufre, METH_NOARGS, 0};
static PyObject *__pyx_pw_10_gappedbuf_1init_gappedbufre(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("init_gappedbufre (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_init_gappedbufre(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_init_gappedbufre(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_sys = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init_gappedbufre", 0);

  /* "_gappedbuf.pyx":15
 * 
 * def init_gappedbufre():
 *     import sys             # <<<<<<<<<<<<<<
 *     sys.modules['_gappedbufre'] = PyInit__gappedbufre()
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_sys = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":16
 * def init_gappedbufre():
 *     import sys
 *     sys.modules['_gappedbufre'] = PyInit__gappedbufre()             # <<<<<<<<<<<<<<
 * 
 * cdef class GappedBuffer:
 */
  __pyx_t_1 = PyInit__gappedbufre(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_sys, __pyx_n_s_modules); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_n_s_gappedbufre, __pyx_t_1) < 0)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":14
 * 
 * 
 * def init_gappedbufre():             # <<<<<<<<<<<<<<
 *     import sys
 *     sys.modules['_gappedbufre'] = PyInit__gappedbufre()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("_gappedbuf.init_gappedbufre", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sys);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":19
 * 
 * cdef class GappedBuffer:
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         DEF BUFSIZE = 4096
 * 
 */

/* Python wrapper */
static int __pyx_pw_10_gappedbuf_12GappedBuffer_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_10_gappedbuf_12GappedBuffer_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer___cinit__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10_gappedbuf_12GappedBuffer___cinit__(struct GappedBufferObj *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "_gappedbuf.pyx":22
 *         DEF BUFSIZE = 4096
 * 
 *         self.buf = <Py_UCS4*>malloc(BUFSIZE * sizeof(Py_UCS4))             # <<<<<<<<<<<<<<
 *         if not self.buf:
 *             raise MemoryError()
 */
  __pyx_v_self->buf = ((Py_UCS4 *)malloc((0x1000 * (sizeof(Py_UCS4)))));

  /* "_gappedbuf.pyx":23
 * 
 *         self.buf = <Py_UCS4*>malloc(BUFSIZE * sizeof(Py_UCS4))
 *         if not self.buf:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_self->buf != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "_gappedbuf.pyx":24
 *         self.buf = <Py_UCS4*>malloc(BUFSIZE * sizeof(Py_UCS4))
 *         if not self.buf:
 *             raise MemoryError()             # <<<<<<<<<<<<<<
 * 
 *         self.bufsize = BUFSIZE
 */
    PyErr_NoMemory(); __PYX_ERR(0, 24, __pyx_L1_error)

    /* "_gappedbuf.pyx":23
 * 
 *         self.buf = <Py_UCS4*>malloc(BUFSIZE * sizeof(Py_UCS4))
 *         if not self.buf:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 * 
 */
  }

  /* "_gappedbuf.pyx":26
 *             raise MemoryError()
 * 
 *         self.bufsize = BUFSIZE             # <<<<<<<<<<<<<<
 *         self.numelems = 0
 *         self.gap = 0
 */
  __pyx_v_self->bufsize = 0x1000;

  /* "_gappedbuf.pyx":27
 * 
 *         self.bufsize = BUFSIZE
 *         self.numelems = 0             # <<<<<<<<<<<<<<
 *         self.gap = 0
 *         self.gapsize = BUFSIZE
 */
  __pyx_v_self->numelems = 0;

  /* "_gappedbuf.pyx":28
 *         self.bufsize = BUFSIZE
 *         self.numelems = 0
 *         self.gap = 0             # <<<<<<<<<<<<<<
 *         self.gapsize = BUFSIZE
 * 
 */
  __pyx_v_self->gap = 0;

  /* "_gappedbuf.pyx":29
 *         self.numelems = 0
 *         self.gap = 0
 *         self.gapsize = BUFSIZE             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->gapsize = 0x1000;

  /* "_gappedbuf.pyx":19
 * 
 * cdef class GappedBuffer:
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         DEF BUFSIZE = 4096
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":31
 *         self.gapsize = BUFSIZE
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.buf:
 *             free(self.buf)
 */

/* Python wrapper */
static void __pyx_pw_10_gappedbuf_12GappedBuffer_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_10_gappedbuf_12GappedBuffer_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_10_gappedbuf_12GappedBuffer_2__dealloc__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_10_gappedbuf_12GappedBuffer_2__dealloc__(struct GappedBufferObj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "_gappedbuf.pyx":32
 * 
 *     def __dealloc__(self):
 *         if self.buf:             # <<<<<<<<<<<<<<
 *             free(self.buf)
 *             self.buf = NULL
 */
  __pyx_t_1 = (__pyx_v_self->buf != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":33
 *     def __dealloc__(self):
 *         if self.buf:
 *             free(self.buf)             # <<<<<<<<<<<<<<
 *             self.buf = NULL
 * 
 */
    free(__pyx_v_self->buf);

    /* "_gappedbuf.pyx":34
 *         if self.buf:
 *             free(self.buf)
 *             self.buf = NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef void _resize_buf(self, Py_ssize_t size):
 */
    __pyx_v_self->buf = NULL;

    /* "_gappedbuf.pyx":32
 * 
 *     def __dealloc__(self):
 *         if self.buf:             # <<<<<<<<<<<<<<
 *             free(self.buf)
 *             self.buf = NULL
 */
  }

  /* "_gappedbuf.pyx":31
 *         self.gapsize = BUFSIZE
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.buf:
 *             free(self.buf)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gappedbuf.pyx":36
 *             self.buf = NULL
 * 
 *     cdef void _resize_buf(self, Py_ssize_t size):             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 *buf = <Py_UCS4*>realloc(
 *                 self.buf, size * sizeof(Py_UCS4))
 */

static void __pyx_f_10_gappedbuf_12GappedBuffer__resize_buf(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_size) {
  Py_UCS4 *__pyx_v_buf;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resize_buf", 0);

  /* "_gappedbuf.pyx":37
 * 
 *     cdef void _resize_buf(self, Py_ssize_t size):
 *         cdef Py_UCS4 *buf = <Py_UCS4*>realloc(             # <<<<<<<<<<<<<<
 *                 self.buf, size * sizeof(Py_UCS4))
 * 
 */
  __pyx_v_buf = ((Py_UCS4 *)realloc(__pyx_v_self->buf, (__pyx_v_size * (sizeof(Py_UCS4)))));

  /* "_gappedbuf.pyx":40
 *                 self.buf, size * sizeof(Py_UCS4))
 * 
 *         if not buf:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 *         self.buf = buf
 */
  __pyx_t_1 = ((!(__pyx_v_buf != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "_gappedbuf.pyx":41
 * 
 *         if not buf:
 *             raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.buf = buf
 *         self.bufsize = size
 */
    PyErr_NoMemory(); __PYX_ERR(0, 41, __pyx_L1_error)

    /* "_gappedbuf.pyx":40
 *                 self.buf, size * sizeof(Py_UCS4))
 * 
 *         if not buf:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 *         self.buf = buf
 */
  }

  /* "_gappedbuf.pyx":42
 *         if not buf:
 *             raise MemoryError()
 *         self.buf = buf             # <<<<<<<<<<<<<<
 *         self.bufsize = size
 * 
 */
  __pyx_v_self->buf = __pyx_v_buf;

  /* "_gappedbuf.pyx":43
 *             raise MemoryError()
 *         self.buf = buf
 *         self.bufsize = size             # <<<<<<<<<<<<<<
 * 
 *     cdef void _expand_gap(self, Py_ssize_t size):
 */
  __pyx_v_self->bufsize = __pyx_v_size;

  /* "_gappedbuf.pyx":36
 *             self.buf = NULL
 * 
 *     cdef void _resize_buf(self, Py_ssize_t size):             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 *buf = <Py_UCS4*>realloc(
 *                 self.buf, size * sizeof(Py_UCS4))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("_gappedbuf.GappedBuffer._resize_buf", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "_gappedbuf.pyx":45
 *         self.bufsize = size
 * 
 *     cdef void _expand_gap(self, Py_ssize_t size):             # <<<<<<<<<<<<<<
 *         if size <= self.gapsize:
 *             return
 */

static void __pyx_f_10_gappedbuf_12GappedBuffer__expand_gap(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_size) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_expand_gap", 0);

  /* "_gappedbuf.pyx":46
 * 
 *     cdef void _expand_gap(self, Py_ssize_t size):
 *         if size <= self.gapsize:             # <<<<<<<<<<<<<<
 *             return
 *         self._resize_buf(self.numelems+size)
 */
  __pyx_t_1 = ((__pyx_v_size <= __pyx_v_self->gapsize) != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":47
 *     cdef void _expand_gap(self, Py_ssize_t size):
 *         if size <= self.gapsize:
 *             return             # <<<<<<<<<<<<<<
 *         self._resize_buf(self.numelems+size)
 * 
 */
    goto __pyx_L0;

    /* "_gappedbuf.pyx":46
 * 
 *     cdef void _expand_gap(self, Py_ssize_t size):
 *         if size <= self.gapsize:             # <<<<<<<<<<<<<<
 *             return
 *         self._resize_buf(self.numelems+size)
 */
  }

  /* "_gappedbuf.pyx":48
 *         if size <= self.gapsize:
 *             return
 *         self._resize_buf(self.numelems+size)             # <<<<<<<<<<<<<<
 * 
 *         if self.gap < self.numelems:
 */
  ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_resize_buf(__pyx_v_self, (__pyx_v_self->numelems + __pyx_v_size));

  /* "_gappedbuf.pyx":50
 *         self._resize_buf(self.numelems+size)
 * 
 *         if self.gap < self.numelems:             # <<<<<<<<<<<<<<
 *             # resize gap
 *             memmove(self.buf+self.gap+size, self.buf+self.gap+self.gapsize,
 */
  __pyx_t_1 = ((__pyx_v_self->gap < __pyx_v_self->numelems) != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":52
 *         if self.gap < self.numelems:
 *             # resize gap
 *             memmove(self.buf+self.gap+size, self.buf+self.gap+self.gapsize,             # <<<<<<<<<<<<<<
 *                     (self.numelems-self.gap) * sizeof(Py_UCS4))
 * 
 */
    (void)(memmove(((__pyx_v_self->buf + __pyx_v_self->gap) + __pyx_v_size), ((__pyx_v_self->buf + __pyx_v_self->gap) + __pyx_v_self->gapsize), ((__pyx_v_self->numelems - __pyx_v_self->gap) * (sizeof(Py_UCS4)))));

    /* "_gappedbuf.pyx":50
 *         self._resize_buf(self.numelems+size)
 * 
 *         if self.gap < self.numelems:             # <<<<<<<<<<<<<<
 *             # resize gap
 *             memmove(self.buf+self.gap+size, self.buf+self.gap+self.gapsize,
 */
  }

  /* "_gappedbuf.pyx":55
 *                     (self.numelems-self.gap) * sizeof(Py_UCS4))
 * 
 *         self.gapsize = size             # <<<<<<<<<<<<<<
 * 
 *     cdef void _shrink_gap(self):
 */
  __pyx_v_self->gapsize = __pyx_v_size;

  /* "_gappedbuf.pyx":45
 *         self.bufsize = size
 * 
 *     cdef void _expand_gap(self, Py_ssize_t size):             # <<<<<<<<<<<<<<
 *         if size <= self.gapsize:
 *             return
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "_gappedbuf.pyx":57
 *         self.gapsize = size
 * 
 *     cdef void _shrink_gap(self):             # <<<<<<<<<<<<<<
 *         DEF MAX_GAPSIZE = 4096
 *         if self.numelems + MAX_GAPSIZE > self.bufsize:
 */

static void __pyx_f_10_gappedbuf_12GappedBuffer__shrink_gap(struct GappedBufferObj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_shrink_gap", 0);

  /* "_gappedbuf.pyx":59
 *     cdef void _shrink_gap(self):
 *         DEF MAX_GAPSIZE = 4096
 *         if self.numelems + MAX_GAPSIZE > self.bufsize:             # <<<<<<<<<<<<<<
 *             return
 *         memmove(self.buf+self.gap+MAX_GAPSIZE,
 */
  __pyx_t_1 = (((__pyx_v_self->numelems + 0x1000) > __pyx_v_self->bufsize) != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":60
 *         DEF MAX_GAPSIZE = 4096
 *         if self.numelems + MAX_GAPSIZE > self.bufsize:
 *             return             # <<<<<<<<<<<<<<
 *         memmove(self.buf+self.gap+MAX_GAPSIZE,
 *                 self.buf+self.gap+self.gapsize,
 */
    goto __pyx_L0;

    /* "_gappedbuf.pyx":59
 *     cdef void _shrink_gap(self):
 *         DEF MAX_GAPSIZE = 4096
 *         if self.numelems + MAX_GAPSIZE > self.bufsize:             # <<<<<<<<<<<<<<
 *             return
 *         memmove(self.buf+self.gap+MAX_GAPSIZE,
 */
  }

  /* "_gappedbuf.pyx":61
 *         if self.numelems + MAX_GAPSIZE > self.bufsize:
 *             return
 *         memmove(self.buf+self.gap+MAX_GAPSIZE,             # <<<<<<<<<<<<<<
 *                 self.buf+self.gap+self.gapsize,
 *                 (self.numelems-self.gap) * sizeof(Py_UCS4))
 */
  (void)(memmove(((__pyx_v_self->buf + __pyx_v_self->gap) + 0x1000), ((__pyx_v_self->buf + __pyx_v_self->gap) + __pyx_v_self->gapsize), ((__pyx_v_self->numelems - __pyx_v_self->gap) * (sizeof(Py_UCS4)))));

  /* "_gappedbuf.pyx":65
 *                 (self.numelems-self.gap) * sizeof(Py_UCS4))
 * 
 *         self.gapsize = MAX_GAPSIZE             # <<<<<<<<<<<<<<
 *         self._resize_buf(self.numelems+MAX_GAPSIZE)
 * 
 */
  __pyx_v_self->gapsize = 0x1000;

  /* "_gappedbuf.pyx":66
 * 
 *         self.gapsize = MAX_GAPSIZE
 *         self._resize_buf(self.numelems+MAX_GAPSIZE)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _move_gap(self, Py_ssize_t index):
 */
  ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_resize_buf(__pyx_v_self, (__pyx_v_self->numelems + 0x1000));

  /* "_gappedbuf.pyx":57
 *         self.gapsize = size
 * 
 *     cdef void _shrink_gap(self):             # <<<<<<<<<<<<<<
 *         DEF MAX_GAPSIZE = 4096
 *         if self.numelems + MAX_GAPSIZE > self.bufsize:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "_gappedbuf.pyx":68
 *         self._resize_buf(self.numelems+MAX_GAPSIZE)
 * 
 *     cdef void _move_gap(self, Py_ssize_t index):             # <<<<<<<<<<<<<<
 *         assert index <= self.bufsize
 *         if self.gap == index:
 */

static void __pyx_f_10_gappedbuf_12GappedBuffer__move_gap(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_move_gap", 0);

  /* "_gappedbuf.pyx":69
 * 
 *     cdef void _move_gap(self, Py_ssize_t index):
 *         assert index <= self.bufsize             # <<<<<<<<<<<<<<
 *         if self.gap == index:
 *             return
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_index <= __pyx_v_self->bufsize) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 69, __pyx_L1_error)
    }
  }
  #endif

  /* "_gappedbuf.pyx":70
 *     cdef void _move_gap(self, Py_ssize_t index):
 *         assert index <= self.bufsize
 *         if self.gap == index:             # <<<<<<<<<<<<<<
 *             return
 *         if self.gapsize:
 */
  __pyx_t_1 = ((__pyx_v_self->gap == __pyx_v_index) != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":71
 *         assert index <= self.bufsize
 *         if self.gap == index:
 *             return             # <<<<<<<<<<<<<<
 *         if self.gapsize:
 *             if index < self.gap:
 */
    goto __pyx_L0;

    /* "_gappedbuf.pyx":70
 *     cdef void _move_gap(self, Py_ssize_t index):
 *         assert index <= self.bufsize
 *         if self.gap == index:             # <<<<<<<<<<<<<<
 *             return
 *         if self.gapsize:
 */
  }

  /* "_gappedbuf.pyx":72
 *         if self.gap == index:
 *             return
 *         if self.gapsize:             # <<<<<<<<<<<<<<
 *             if index < self.gap:
 *                 memmove(self.buf+index+self.gapsize, self.buf+index,
 */
  __pyx_t_1 = (__pyx_v_self->gapsize != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":73
 *             return
 *         if self.gapsize:
 *             if index < self.gap:             # <<<<<<<<<<<<<<
 *                 memmove(self.buf+index+self.gapsize, self.buf+index,
 *                         (self.gap - index)*sizeof(Py_UCS4))
 */
    __pyx_t_1 = ((__pyx_v_index < __pyx_v_self->gap) != 0);
    if (__pyx_t_1) {

      /* "_gappedbuf.pyx":74
 *         if self.gapsize:
 *             if index < self.gap:
 *                 memmove(self.buf+index+self.gapsize, self.buf+index,             # <<<<<<<<<<<<<<
 *                         (self.gap - index)*sizeof(Py_UCS4))
 *             else:
 */
      (void)(memmove(((__pyx_v_self->buf + __pyx_v_index) + __pyx_v_self->gapsize), (__pyx_v_self->buf + __pyx_v_index), ((__pyx_v_self->gap - __pyx_v_index) * (sizeof(Py_UCS4)))));

      /* "_gappedbuf.pyx":73
 *             return
 *         if self.gapsize:
 *             if index < self.gap:             # <<<<<<<<<<<<<<
 *                 memmove(self.buf+index+self.gapsize, self.buf+index,
 *                         (self.gap - index)*sizeof(Py_UCS4))
 */
      goto __pyx_L5;
    }

    /* "_gappedbuf.pyx":77
 *                         (self.gap - index)*sizeof(Py_UCS4))
 *             else:
 *                 memmove(self.buf+self.gap, self.buf+self.gap+self.gapsize,             # <<<<<<<<<<<<<<
 *                         (index - self.gap)*sizeof(Py_UCS4))
 *         self.gap = index
 */
    /*else*/ {

      /* "_gappedbuf.pyx":78
 *             else:
 *                 memmove(self.buf+self.gap, self.buf+self.gap+self.gapsize,
 *                         (index - self.gap)*sizeof(Py_UCS4))             # <<<<<<<<<<<<<<
 *         self.gap = index
 * 
 */
      (void)(memmove((__pyx_v_self->buf + __pyx_v_self->gap), ((__pyx_v_self->buf + __pyx_v_self->gap) + __pyx_v_self->gapsize), ((__pyx_v_index - __pyx_v_self->gap) * (sizeof(Py_UCS4)))));
    }
    __pyx_L5:;

    /* "_gappedbuf.pyx":72
 *         if self.gap == index:
 *             return
 *         if self.gapsize:             # <<<<<<<<<<<<<<
 *             if index < self.gap:
 *                 memmove(self.buf+index+self.gapsize, self.buf+index,
 */
  }

  /* "_gappedbuf.pyx":79
 *                 memmove(self.buf+self.gap, self.buf+self.gap+self.gapsize,
 *                         (index - self.gap)*sizeof(Py_UCS4))
 *         self.gap = index             # <<<<<<<<<<<<<<
 * 
 *     cdef _insert(self, Py_ssize_t index, unicode s):
 */
  __pyx_v_self->gap = __pyx_v_index;

  /* "_gappedbuf.pyx":68
 *         self._resize_buf(self.numelems+MAX_GAPSIZE)
 * 
 *     cdef void _move_gap(self, Py_ssize_t index):             # <<<<<<<<<<<<<<
 *         assert index <= self.bufsize
 *         if self.gap == index:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("_gappedbuf.GappedBuffer._move_gap", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "_gappedbuf.pyx":81
 *         self.gap = index
 * 
 *     cdef _insert(self, Py_ssize_t index, unicode s):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):
 */

static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer__insert(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s) {
  Py_ssize_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  long __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_UCS4 *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_insert", 0);

  /* "_gappedbuf.pyx":82
 * 
 *     cdef _insert(self, Py_ssize_t index, unicode s):
 *         cdef Py_ssize_t size = len(s)             # <<<<<<<<<<<<<<
 *         if not (0 <= index <= self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (
 */
  if (unlikely(__pyx_v_s == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 82, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_s); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 82, __pyx_L1_error)
  __pyx_v_size = __pyx_t_1;

  /* "_gappedbuf.pyx":83
 *     cdef _insert(self, Py_ssize_t index, unicode s):
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 index, self.numelems))
 */
  __pyx_t_2 = (0 <= __pyx_v_index);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_index <= __pyx_v_self->numelems);
  }
  __pyx_t_3 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "_gappedbuf.pyx":85
 *         if not (0 <= index <= self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 index, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         if size:
 */
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":84
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 index, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_index_value_d_d, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 84, __pyx_L1_error)

    /* "_gappedbuf.pyx":83
 *     cdef _insert(self, Py_ssize_t index, unicode s):
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 index, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":87
 *                                 index, self.numelems))
 * 
 *         if size:             # <<<<<<<<<<<<<<
 *             self._move_gap(index)
 *             if self.gapsize < size:
 */
  __pyx_t_3 = (__pyx_v_size != 0);
  if (__pyx_t_3) {

    /* "_gappedbuf.pyx":88
 * 
 *         if size:
 *             self._move_gap(index)             # <<<<<<<<<<<<<<
 *             if self.gapsize < size:
 *                 self._expand_gap(max(1024, size*2))
 */
    ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_move_gap(__pyx_v_self, __pyx_v_index);

    /* "_gappedbuf.pyx":89
 *         if size:
 *             self._move_gap(index)
 *             if self.gapsize < size:             # <<<<<<<<<<<<<<
 *                 self._expand_gap(max(1024, size*2))
 *             rtdef.PyUnicode_AsUCS4(s, self.buf+index, size, False)
 */
    __pyx_t_3 = ((__pyx_v_self->gapsize < __pyx_v_size) != 0);
    if (__pyx_t_3) {

      /* "_gappedbuf.pyx":90
 *             self._move_gap(index)
 *             if self.gapsize < size:
 *                 self._expand_gap(max(1024, size*2))             # <<<<<<<<<<<<<<
 *             rtdef.PyUnicode_AsUCS4(s, self.buf+index, size, False)
 * 
 */
      __pyx_t_1 = (__pyx_v_size * 2);
      __pyx_t_7 = 0x400;
      if (((__pyx_t_1 > __pyx_t_7) != 0)) {
        __pyx_t_8 = __pyx_t_1;
      } else {
        __pyx_t_8 = __pyx_t_7;
      }
      ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_expand_gap(__pyx_v_self, __pyx_t_8);

      /* "_gappedbuf.pyx":89
 *         if size:
 *             self._move_gap(index)
 *             if self.gapsize < size:             # <<<<<<<<<<<<<<
 *                 self._expand_gap(max(1024, size*2))
 *             rtdef.PyUnicode_AsUCS4(s, self.buf+index, size, False)
 */
    }

    /* "_gappedbuf.pyx":91
 *             if self.gapsize < size:
 *                 self._expand_gap(max(1024, size*2))
 *             rtdef.PyUnicode_AsUCS4(s, self.buf+index, size, False)             # <<<<<<<<<<<<<<
 * 
 *             self.numelems += size
 */
    __pyx_t_9 = PyUnicode_AsUCS4(__pyx_v_s, (__pyx_v_self->buf + __pyx_v_index), __pyx_v_size, 0); if (unlikely(__pyx_t_9 == ((Py_UCS4 *)NULL))) __PYX_ERR(0, 91, __pyx_L1_error)

    /* "_gappedbuf.pyx":93
 *             rtdef.PyUnicode_AsUCS4(s, self.buf+index, size, False)
 * 
 *             self.numelems += size             # <<<<<<<<<<<<<<
 *             self.gap += size
 *             self.gapsize -= size
 */
    __pyx_v_self->numelems = (__pyx_v_self->numelems + __pyx_v_size);

    /* "_gappedbuf.pyx":94
 * 
 *             self.numelems += size
 *             self.gap += size             # <<<<<<<<<<<<<<
 *             self.gapsize -= size
 * 
 */
    __pyx_v_self->gap = (__pyx_v_self->gap + __pyx_v_size);

    /* "_gappedbuf.pyx":95
 *             self.numelems += size
 *             self.gap += size
 *             self.gapsize -= size             # <<<<<<<<<<<<<<
 * 
 *     cpdef insert(self, Py_ssize_t index, unicode s):
 */
    __pyx_v_self->gapsize = (__pyx_v_self->gapsize - __pyx_v_size);

    /* "_gappedbuf.pyx":87
 *                                 index, self.numelems))
 * 
 *         if size:             # <<<<<<<<<<<<<<
 *             self._move_gap(index)
 *             if self.gapsize < size:
 */
  }

  /* "_gappedbuf.pyx":81
 *         self.gap = index
 * 
 *     cdef _insert(self, Py_ssize_t index, unicode s):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer._insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":97
 *             self.gapsize -= size
 * 
 *     cpdef insert(self, Py_ssize_t index, unicode s):             # <<<<<<<<<<<<<<
 *         self._insert(index, s)
 * 
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_5insert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_insert(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_insert); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_5insert)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 97, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 97, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
          __Pyx_INCREF(__pyx_v_s);
          __Pyx_GIVEREF(__pyx_v_s);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_s);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":98
 * 
 *     cpdef insert(self, Py_ssize_t index, unicode s):
 *         self._insert(index, s)             # <<<<<<<<<<<<<<
 * 
 *     cpdef append(self, unicode s):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_insert(__pyx_v_self, __pyx_v_index, __pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":97
 *             self.gapsize -= size
 * 
 *     cpdef insert(self, Py_ssize_t index, unicode s):             # <<<<<<<<<<<<<<
 *         self._insert(index, s)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_5insert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_5insert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_index;
  PyObject *__pyx_v_s = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_s,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_s)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("insert", 1, 2, 2, 1); __PYX_ERR(0, 97, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "insert") < 0)) __PYX_ERR(0, 97, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_index == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 97, __pyx_L3_error)
    __pyx_v_s = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insert", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 97, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_s), (&PyUnicode_Type), 1, "s", 1))) __PYX_ERR(0, 97, __pyx_L1_error)
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_4insert(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_index, __pyx_v_s);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_4insert(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_insert(__pyx_v_self, __pyx_v_index, __pyx_v_s, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":100
 *         self._insert(index, s)
 * 
 *     cpdef append(self, unicode s):             # <<<<<<<<<<<<<<
 *         self.insert(self.numelems, s)
 * 
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_7append(PyObject *__pyx_v_self, PyObject *__pyx_v_s); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_append(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("append", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_append); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_7append)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_s) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_s);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":101
 * 
 *     cpdef append(self, unicode s):
 *         self.insert(self.numelems, s)             # <<<<<<<<<<<<<<
 * 
 *     cdef _delete(self, Py_ssize_t begin, Py_ssize_t end):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->insert(__pyx_v_self, __pyx_v_self->numelems, __pyx_v_s, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":100
 *         self._insert(index, s)
 * 
 *     cpdef append(self, unicode s):             # <<<<<<<<<<<<<<
 *         self.insert(self.numelems, s)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_7append(PyObject *__pyx_v_self, PyObject *__pyx_v_s); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_7append(PyObject *__pyx_v_self, PyObject *__pyx_v_s) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("append (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_s), (&PyUnicode_Type), 1, "s", 1))) __PYX_ERR(0, 100, __pyx_L1_error)
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_6append(((struct GappedBufferObj *)__pyx_v_self), ((PyObject*)__pyx_v_s));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_6append(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("append", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_append(__pyx_v_self, __pyx_v_s, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":103
 *         self.insert(self.numelems, s)
 * 
 *     cdef _delete(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 */

static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer__delete(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  Py_ssize_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_delete", 0);

  /* "_gappedbuf.pyx":104
 * 
 *     cdef _delete(self, Py_ssize_t begin, Py_ssize_t end):
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_1 = (0 <= __pyx_v_begin);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "_gappedbuf.pyx":106
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         cdef Py_ssize_t size = end-begin
 */
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":105
 *     cdef _delete(self, Py_ssize_t begin, Py_ssize_t end):
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 105, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 105, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 105, __pyx_L1_error)

    /* "_gappedbuf.pyx":104
 * 
 *     cdef _delete(self, Py_ssize_t begin, Py_ssize_t end):
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":108
 *                                 begin, end, self.numelems))
 * 
 *         cdef Py_ssize_t size = end-begin             # <<<<<<<<<<<<<<
 *         if not size:
 *             return
 */
  __pyx_v_size = (__pyx_v_end - __pyx_v_begin);

  /* "_gappedbuf.pyx":109
 * 
 *         cdef Py_ssize_t size = end-begin
 *         if not size:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_size != 0)) != 0);
  if (__pyx_t_2) {

    /* "_gappedbuf.pyx":110
 *         cdef Py_ssize_t size = end-begin
 *         if not size:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         self._move_gap(begin)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "_gappedbuf.pyx":109
 * 
 *         cdef Py_ssize_t size = end-begin
 *         if not size:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  }

  /* "_gappedbuf.pyx":112
 *             return
 * 
 *         self._move_gap(begin)             # <<<<<<<<<<<<<<
 *         self.gapsize += size
 *         self.numelems -= size
 */
  ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_move_gap(__pyx_v_self, __pyx_v_begin);

  /* "_gappedbuf.pyx":113
 * 
 *         self._move_gap(begin)
 *         self.gapsize += size             # <<<<<<<<<<<<<<
 *         self.numelems -= size
 *         self._shrink_gap()
 */
  __pyx_v_self->gapsize = (__pyx_v_self->gapsize + __pyx_v_size);

  /* "_gappedbuf.pyx":114
 *         self._move_gap(begin)
 *         self.gapsize += size
 *         self.numelems -= size             # <<<<<<<<<<<<<<
 *         self._shrink_gap()
 * 
 */
  __pyx_v_self->numelems = (__pyx_v_self->numelems - __pyx_v_size);

  /* "_gappedbuf.pyx":115
 *         self.gapsize += size
 *         self.numelems -= size
 *         self._shrink_gap()             # <<<<<<<<<<<<<<
 * 
 *     cpdef delete(self, Py_ssize_t begin, Py_ssize_t end):
 */
  ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_shrink_gap(__pyx_v_self);

  /* "_gappedbuf.pyx":103
 *         self.insert(self.numelems, s)
 * 
 *     cdef _delete(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer._delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":117
 *         self._shrink_gap()
 * 
 *     cpdef delete(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         self._delete(begin, end)
 * 
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_9delete(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_delete(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delete); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_9delete)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 117, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 117, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 117, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":118
 * 
 *     cpdef delete(self, Py_ssize_t begin, Py_ssize_t end):
 *         self._delete(begin, end)             # <<<<<<<<<<<<<<
 * 
 *     cpdef replace(self, Py_ssize_t begin, Py_ssize_t end, unicode s):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_delete(__pyx_v_self, __pyx_v_begin, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":117
 *         self._shrink_gap()
 * 
 *     cpdef delete(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         self._delete(begin, end)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_9delete(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_9delete(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("delete", 1, 2, 2, 1); __PYX_ERR(0, 117, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "delete") < 0)) __PYX_ERR(0, 117, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 117, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_8delete(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_begin, __pyx_v_end);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_8delete(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_delete(__pyx_v_self, __pyx_v_begin, __pyx_v_end, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":120
 *         self._delete(begin, end)
 * 
 *     cpdef replace(self, Py_ssize_t begin, Py_ssize_t end, unicode s):             # <<<<<<<<<<<<<<
 *         self._delete(begin, end)
 *         self._insert(begin, s)
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_11replace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_replace(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("replace", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_replace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_11replace)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 120, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 120, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 120, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __Pyx_INCREF(__pyx_v_s);
          __Pyx_GIVEREF(__pyx_v_s);
          PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_v_s);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":121
 * 
 *     cpdef replace(self, Py_ssize_t begin, Py_ssize_t end, unicode s):
 *         self._delete(begin, end)             # <<<<<<<<<<<<<<
 *         self._insert(begin, s)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_delete(__pyx_v_self, __pyx_v_begin, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":122
 *     cpdef replace(self, Py_ssize_t begin, Py_ssize_t end, unicode s):
 *         self._delete(begin, end)
 *         self._insert(begin, s)             # <<<<<<<<<<<<<<
 * 
 *     cdef Py_UCS4 _get_max(self, Py_ssize_t begin, Py_ssize_t end) nogil:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_insert(__pyx_v_self, __pyx_v_begin, __pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":120
 *         self._delete(begin, end)
 * 
 *     cpdef replace(self, Py_ssize_t begin, Py_ssize_t end, unicode s):             # <<<<<<<<<<<<<<
 *         self._delete(begin, end)
 *         self._insert(begin, s)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.replace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_11replace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_11replace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_s = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("replace (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,&__pyx_n_s_s,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("replace", 1, 3, 3, 1); __PYX_ERR(0, 120, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_s)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("replace", 1, 3, 3, 2); __PYX_ERR(0, 120, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "replace") < 0)) __PYX_ERR(0, 120, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 120, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 120, __pyx_L3_error)
    __pyx_v_s = ((PyObject*)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("replace", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 120, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.replace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_s), (&PyUnicode_Type), 1, "s", 1))) __PYX_ERR(0, 120, __pyx_L1_error)
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_10replace(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_begin, __pyx_v_end, __pyx_v_s);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_10replace(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("replace", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_replace(__pyx_v_self, __pyx_v_begin, __pyx_v_end, __pyx_v_s, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.replace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":124
 *         self._insert(begin, s)
 * 
 *     cdef Py_UCS4 _get_max(self, Py_ssize_t begin, Py_ssize_t end) nogil:             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 maxchar = 0
 *         cdef Py_ssize_t b, e
 */

static Py_UCS4 __pyx_f_10_gappedbuf_12GappedBuffer__get_max(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  Py_UCS4 __pyx_v_maxchar;
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  Py_UCS4 __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_UCS4 __pyx_t_4;
  Py_UCS4 __pyx_t_5;
  Py_UCS4 __pyx_t_6;

  /* "_gappedbuf.pyx":125
 * 
 *     cdef Py_UCS4 _get_max(self, Py_ssize_t begin, Py_ssize_t end) nogil:
 *         cdef Py_UCS4 maxchar = 0             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t b, e
 * 
 */
  __pyx_v_maxchar = 0;

  /* "_gappedbuf.pyx":128
 *         cdef Py_ssize_t b, e
 * 
 *         e = min(end, self.gap)             # <<<<<<<<<<<<<<
 * 
 *         cdef Py_ssize_t p
 */
  __pyx_t_1 = __pyx_v_self->gap;
  __pyx_t_2 = __pyx_v_end;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_e = __pyx_t_3;

  /* "_gappedbuf.pyx":131
 * 
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:             # <<<<<<<<<<<<<<
 *             maxchar = max(maxchar, self.buf[p])
 *         b = max(begin, self.gap)
 */
  __pyx_t_3 = __pyx_v_e;
  for (__pyx_v_p = __pyx_v_begin; __pyx_v_p < __pyx_t_3; __pyx_v_p++) {

    /* "_gappedbuf.pyx":132
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:
 *             maxchar = max(maxchar, self.buf[p])             # <<<<<<<<<<<<<<
 *         b = max(begin, self.gap)
 *         for p from b <= p < end:
 */
    __pyx_t_4 = (__pyx_v_self->buf[__pyx_v_p]);
    __pyx_t_5 = __pyx_v_maxchar;
    if (((__pyx_t_4 > __pyx_t_5) != 0)) {
      __pyx_t_6 = __pyx_t_4;
    } else {
      __pyx_t_6 = __pyx_t_5;
    }
    __pyx_v_maxchar = __pyx_t_6;
  }

  /* "_gappedbuf.pyx":133
 *         for p from begin <= p < e:
 *             maxchar = max(maxchar, self.buf[p])
 *         b = max(begin, self.gap)             # <<<<<<<<<<<<<<
 *         for p from b <= p < end:
 *             maxchar = max(maxchar, self.buf[p+self.gapsize])
 */
  __pyx_t_3 = __pyx_v_self->gap;
  __pyx_t_1 = __pyx_v_begin;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_b = __pyx_t_2;

  /* "_gappedbuf.pyx":134
 *             maxchar = max(maxchar, self.buf[p])
 *         b = max(begin, self.gap)
 *         for p from b <= p < end:             # <<<<<<<<<<<<<<
 *             maxchar = max(maxchar, self.buf[p+self.gapsize])
 *         return maxchar
 */
  __pyx_t_2 = __pyx_v_end;
  for (__pyx_v_p = __pyx_v_b; __pyx_v_p < __pyx_t_2; __pyx_v_p++) {

    /* "_gappedbuf.pyx":135
 *         b = max(begin, self.gap)
 *         for p from b <= p < end:
 *             maxchar = max(maxchar, self.buf[p+self.gapsize])             # <<<<<<<<<<<<<<
 *         return maxchar
 * 
 */
    __pyx_t_6 = (__pyx_v_self->buf[(__pyx_v_p + __pyx_v_self->gapsize)]);
    __pyx_t_4 = __pyx_v_maxchar;
    if (((__pyx_t_6 > __pyx_t_4) != 0)) {
      __pyx_t_5 = __pyx_t_6;
    } else {
      __pyx_t_5 = __pyx_t_4;
    }
    __pyx_v_maxchar = __pyx_t_5;
  }

  /* "_gappedbuf.pyx":136
 *         for p from b <= p < end:
 *             maxchar = max(maxchar, self.buf[p+self.gapsize])
 *         return maxchar             # <<<<<<<<<<<<<<
 * 
 *     cdef void _copy_chars1(self, void *dest, Py_ssize_t begin,
 */
  __pyx_r = __pyx_v_maxchar;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":124
 *         self._insert(begin, s)
 * 
 *     cdef Py_UCS4 _get_max(self, Py_ssize_t begin, Py_ssize_t end) nogil:             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 maxchar = 0
 *         cdef Py_ssize_t b, e
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_gappedbuf.pyx":138
 *         return maxchar
 * 
 *     cdef void _copy_chars1(self, void *dest, Py_ssize_t begin,             # <<<<<<<<<<<<<<
 *             Py_ssize_t end) nogil:
 *         cdef rtdef.Py_UCS1 *buf = <rtdef.Py_UCS1 *>dest
 */

static void __pyx_f_10_gappedbuf_12GappedBuffer__copy_chars1(struct GappedBufferObj *__pyx_v_self, void *__pyx_v_dest, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  Py_UCS1 *__pyx_v_buf;
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;

  /* "_gappedbuf.pyx":140
 *     cdef void _copy_chars1(self, void *dest, Py_ssize_t begin,
 *             Py_ssize_t end) nogil:
 *         cdef rtdef.Py_UCS1 *buf = <rtdef.Py_UCS1 *>dest             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t b, e
 * 
 */
  __pyx_v_buf = ((Py_UCS1 *)__pyx_v_dest);

  /* "_gappedbuf.pyx":144
 * 
 *         # copy before gap
 *         e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:
 */
  __pyx_t_1 = __pyx_v_self->gap;
  __pyx_t_2 = __pyx_v_end;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_e = __pyx_t_3;

  /* "_gappedbuf.pyx":146
 *         e = min(end, self.gap)
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:             # <<<<<<<<<<<<<<
 *             buf[0] = self.buf[p]
 *             buf += 1
 */
  __pyx_t_3 = __pyx_v_e;
  for (__pyx_v_p = __pyx_v_begin; __pyx_v_p < __pyx_t_3; __pyx_v_p++) {

    /* "_gappedbuf.pyx":147
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:
 *             buf[0] = self.buf[p]             # <<<<<<<<<<<<<<
 *             buf += 1
 * 
 */
    (__pyx_v_buf[0]) = (__pyx_v_self->buf[__pyx_v_p]);

    /* "_gappedbuf.pyx":148
 *         for p from begin <= p < e:
 *             buf[0] = self.buf[p]
 *             buf += 1             # <<<<<<<<<<<<<<
 * 
 *         # copy after gap
 */
    __pyx_v_buf = (__pyx_v_buf + 1);
  }

  /* "_gappedbuf.pyx":151
 * 
 *         # copy after gap
 *         b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *         for p from b <= p < end+self.gapsize:
 *             buf[0] = self.buf[p]
 */
  __pyx_t_3 = __pyx_v_self->gap;
  __pyx_t_1 = __pyx_v_begin;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_b = (__pyx_t_2 + __pyx_v_self->gapsize);

  /* "_gappedbuf.pyx":152
 *         # copy after gap
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:             # <<<<<<<<<<<<<<
 *             buf[0] = self.buf[p]
 *             buf += 1
 */
  __pyx_t_2 = (__pyx_v_end + __pyx_v_self->gapsize);
  for (__pyx_v_p = __pyx_v_b; __pyx_v_p < __pyx_t_2; __pyx_v_p++) {

    /* "_gappedbuf.pyx":153
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:
 *             buf[0] = self.buf[p]             # <<<<<<<<<<<<<<
 *             buf += 1
 * 
 */
    (__pyx_v_buf[0]) = (__pyx_v_self->buf[__pyx_v_p]);

    /* "_gappedbuf.pyx":154
 *         for p from b <= p < end+self.gapsize:
 *             buf[0] = self.buf[p]
 *             buf += 1             # <<<<<<<<<<<<<<
 * 
 *     cdef void _copy_chars2(self, void *dest, Py_ssize_t begin,
 */
    __pyx_v_buf = (__pyx_v_buf + 1);
  }

  /* "_gappedbuf.pyx":138
 *         return maxchar
 * 
 *     cdef void _copy_chars1(self, void *dest, Py_ssize_t begin,             # <<<<<<<<<<<<<<
 *             Py_ssize_t end) nogil:
 *         cdef rtdef.Py_UCS1 *buf = <rtdef.Py_UCS1 *>dest
 */

  /* function exit code */
}

/* "_gappedbuf.pyx":156
 *             buf += 1
 * 
 *     cdef void _copy_chars2(self, void *dest, Py_ssize_t begin,             # <<<<<<<<<<<<<<
 *             Py_ssize_t end) nogil:
 *         cdef rtdef.Py_UCS2 *buf = <rtdef.Py_UCS2 *>dest
 */

static void __pyx_f_10_gappedbuf_12GappedBuffer__copy_chars2(struct GappedBufferObj *__pyx_v_self, void *__pyx_v_dest, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  Py_UCS2 *__pyx_v_buf;
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;

  /* "_gappedbuf.pyx":158
 *     cdef void _copy_chars2(self, void *dest, Py_ssize_t begin,
 *             Py_ssize_t end) nogil:
 *         cdef rtdef.Py_UCS2 *buf = <rtdef.Py_UCS2 *>dest             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t b, e
 * 
 */
  __pyx_v_buf = ((Py_UCS2 *)__pyx_v_dest);

  /* "_gappedbuf.pyx":162
 * 
 *         # copy before gap
 *         e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:
 */
  __pyx_t_1 = __pyx_v_self->gap;
  __pyx_t_2 = __pyx_v_end;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_e = __pyx_t_3;

  /* "_gappedbuf.pyx":164
 *         e = min(end, self.gap)
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:             # <<<<<<<<<<<<<<
 *             buf[0] = self.buf[p]
 *             buf += 1
 */
  __pyx_t_3 = __pyx_v_e;
  for (__pyx_v_p = __pyx_v_begin; __pyx_v_p < __pyx_t_3; __pyx_v_p++) {

    /* "_gappedbuf.pyx":165
 *         cdef Py_ssize_t p
 *         for p from begin <= p < e:
 *             buf[0] = self.buf[p]             # <<<<<<<<<<<<<<
 *             buf += 1
 * 
 */
    (__pyx_v_buf[0]) = (__pyx_v_self->buf[__pyx_v_p]);

    /* "_gappedbuf.pyx":166
 *         for p from begin <= p < e:
 *             buf[0] = self.buf[p]
 *             buf += 1             # <<<<<<<<<<<<<<
 * 
 *         # copy after gap
 */
    __pyx_v_buf = (__pyx_v_buf + 1);
  }

  /* "_gappedbuf.pyx":169
 * 
 *         # copy after gap
 *         b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *         for p from b <= p < end+self.gapsize:
 *             buf[0] = self.buf[p]
 */
  __pyx_t_3 = __pyx_v_self->gap;
  __pyx_t_1 = __pyx_v_begin;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_b = (__pyx_t_2 + __pyx_v_self->gapsize);

  /* "_gappedbuf.pyx":170
 *         # copy after gap
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:             # <<<<<<<<<<<<<<
 *             buf[0] = self.buf[p]
 *             buf += 1
 */
  __pyx_t_2 = (__pyx_v_end + __pyx_v_self->gapsize);
  for (__pyx_v_p = __pyx_v_b; __pyx_v_p < __pyx_t_2; __pyx_v_p++) {

    /* "_gappedbuf.pyx":171
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:
 *             buf[0] = self.buf[p]             # <<<<<<<<<<<<<<
 *             buf += 1
 * 
 */
    (__pyx_v_buf[0]) = (__pyx_v_self->buf[__pyx_v_p]);

    /* "_gappedbuf.pyx":172
 *         for p from b <= p < end+self.gapsize:
 *             buf[0] = self.buf[p]
 *             buf += 1             # <<<<<<<<<<<<<<
 * 
 *     cdef void _copy_chars4(self, void *dest, Py_ssize_t begin,
 */
    __pyx_v_buf = (__pyx_v_buf + 1);
  }

  /* "_gappedbuf.pyx":156
 *             buf += 1
 * 
 *     cdef void _copy_chars2(self, void *dest, Py_ssize_t begin,             # <<<<<<<<<<<<<<
 *             Py_ssize_t end) nogil:
 *         cdef rtdef.Py_UCS2 *buf = <rtdef.Py_UCS2 *>dest
 */

  /* function exit code */
}

/* "_gappedbuf.pyx":174
 *             buf += 1
 * 
 *     cdef void _copy_chars4(self, void *dest, Py_ssize_t begin,             # <<<<<<<<<<<<<<
 *             Py_ssize_t end) nogil:
 *         cdef Py_ssize_t n = end - begin
 */

static void __pyx_f_10_gappedbuf_12GappedBuffer__copy_chars4(struct GappedBufferObj *__pyx_v_self, void *__pyx_v_dest, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  Py_ssize_t __pyx_v_n;
  Py_ssize_t __pyx_v_former;
  int __pyx_t_1;

  /* "_gappedbuf.pyx":176
 *     cdef void _copy_chars4(self, void *dest, Py_ssize_t begin,
 *             Py_ssize_t end) nogil:
 *         cdef Py_ssize_t n = end - begin             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t former
 * 
 */
  __pyx_v_n = (__pyx_v_end - __pyx_v_begin);

  /* "_gappedbuf.pyx":179
 *         cdef Py_ssize_t former
 * 
 *         if self.gap < begin:             # <<<<<<<<<<<<<<
 *             memcpy(dest, self.buf+begin+self.gapsize, n*sizeof(Py_UCS4))
 *         elif end <= self.gap:
 */
  __pyx_t_1 = ((__pyx_v_self->gap < __pyx_v_begin) != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":180
 * 
 *         if self.gap < begin:
 *             memcpy(dest, self.buf+begin+self.gapsize, n*sizeof(Py_UCS4))             # <<<<<<<<<<<<<<
 *         elif end <= self.gap:
 *             memcpy(dest, self.buf+begin, n*sizeof(Py_UCS4))
 */
    (void)(memcpy(__pyx_v_dest, ((__pyx_v_self->buf + __pyx_v_begin) + __pyx_v_self->gapsize), (__pyx_v_n * (sizeof(Py_UCS4)))));

    /* "_gappedbuf.pyx":179
 *         cdef Py_ssize_t former
 * 
 *         if self.gap < begin:             # <<<<<<<<<<<<<<
 *             memcpy(dest, self.buf+begin+self.gapsize, n*sizeof(Py_UCS4))
 *         elif end <= self.gap:
 */
    goto __pyx_L3;
  }

  /* "_gappedbuf.pyx":181
 *         if self.gap < begin:
 *             memcpy(dest, self.buf+begin+self.gapsize, n*sizeof(Py_UCS4))
 *         elif end <= self.gap:             # <<<<<<<<<<<<<<
 *             memcpy(dest, self.buf+begin, n*sizeof(Py_UCS4))
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_end <= __pyx_v_self->gap) != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":182
 *             memcpy(dest, self.buf+begin+self.gapsize, n*sizeof(Py_UCS4))
 *         elif end <= self.gap:
 *             memcpy(dest, self.buf+begin, n*sizeof(Py_UCS4))             # <<<<<<<<<<<<<<
 *         else:
 *             former = self.gap - begin
 */
    (void)(memcpy(__pyx_v_dest, (__pyx_v_self->buf + __pyx_v_begin), (__pyx_v_n * (sizeof(Py_UCS4)))));

    /* "_gappedbuf.pyx":181
 *         if self.gap < begin:
 *             memcpy(dest, self.buf+begin+self.gapsize, n*sizeof(Py_UCS4))
 *         elif end <= self.gap:             # <<<<<<<<<<<<<<
 *             memcpy(dest, self.buf+begin, n*sizeof(Py_UCS4))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "_gappedbuf.pyx":184
 *             memcpy(dest, self.buf+begin, n*sizeof(Py_UCS4))
 *         else:
 *             former = self.gap - begin             # <<<<<<<<<<<<<<
 *             memcpy(dest, self.buf+begin, former*sizeof(Py_UCS4))
 *             memcpy(dest+former*sizeof(Py_UCS4), self.buf+self.gap+self.gapsize,
 */
  /*else*/ {
    __pyx_v_former = (__pyx_v_self->gap - __pyx_v_begin);

    /* "_gappedbuf.pyx":185
 *         else:
 *             former = self.gap - begin
 *             memcpy(dest, self.buf+begin, former*sizeof(Py_UCS4))             # <<<<<<<<<<<<<<
 *             memcpy(dest+former*sizeof(Py_UCS4), self.buf+self.gap+self.gapsize,
 *                     (n-former)*sizeof(Py_UCS4))
 */
    (void)(memcpy(__pyx_v_dest, (__pyx_v_self->buf + __pyx_v_begin), (__pyx_v_former * (sizeof(Py_UCS4)))));

    /* "_gappedbuf.pyx":186
 *             former = self.gap - begin
 *             memcpy(dest, self.buf+begin, former*sizeof(Py_UCS4))
 *             memcpy(dest+former*sizeof(Py_UCS4), self.buf+self.gap+self.gapsize,             # <<<<<<<<<<<<<<
 *                     (n-former)*sizeof(Py_UCS4))
 * 
 */
    (void)(memcpy((__pyx_v_dest + (__pyx_v_former * (sizeof(Py_UCS4)))), ((__pyx_v_self->buf + __pyx_v_self->gap) + __pyx_v_self->gapsize), ((__pyx_v_n - __pyx_v_former) * (sizeof(Py_UCS4)))));
  }
  __pyx_L3:;

  /* "_gappedbuf.pyx":174
 *             buf += 1
 * 
 *     cdef void _copy_chars4(self, void *dest, Py_ssize_t begin,             # <<<<<<<<<<<<<<
 *             Py_ssize_t end) nogil:
 *         cdef Py_ssize_t n = end - begin
 */

  /* function exit code */
}

/* "_gappedbuf.pyx":189
 *                     (n-former)*sizeof(Py_UCS4))
 * 
 *     cpdef get(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise IndexError('Invalid range: %d, %d, %d' % (
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_13get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_get(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_skip_dispatch) {
  Py_ssize_t __pyx_v_n;
  Py_UCS4 __pyx_v_maxchar;
  PyObject *__pyx_v_ret = 0;
  void *__pyx_v_buf;
  PyObject *__pyx_v_nbytes = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  void *__pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_13get)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 189, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 189, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":190
 * 
 *     cpdef get(self, Py_ssize_t begin, Py_ssize_t end):
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise IndexError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_9 = (0 <= __pyx_v_begin);
  if (__pyx_t_9) {
    __pyx_t_9 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_9) {
      __pyx_t_9 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_10 = ((!(__pyx_t_9 != 0)) != 0);
  if (unlikely(__pyx_t_10)) {

    /* "_gappedbuf.pyx":192
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise IndexError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         cdef Py_ssize_t n = end - begin
 */
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_5);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":191
 *     cpdef get(self, Py_ssize_t begin, Py_ssize_t end):
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise IndexError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 191, __pyx_L1_error)

    /* "_gappedbuf.pyx":190
 * 
 *     cpdef get(self, Py_ssize_t begin, Py_ssize_t end):
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise IndexError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":194
 *                                 begin, end, self.numelems))
 * 
 *         cdef Py_ssize_t n = end - begin             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 maxchar = self._get_max(begin, end)
 *         cdef unicode ret = rtdef.PyUnicode_New(n, maxchar)
 */
  __pyx_v_n = (__pyx_v_end - __pyx_v_begin);

  /* "_gappedbuf.pyx":195
 * 
 *         cdef Py_ssize_t n = end - begin
 *         cdef Py_UCS4 maxchar = self._get_max(begin, end)             # <<<<<<<<<<<<<<
 *         cdef unicode ret = rtdef.PyUnicode_New(n, maxchar)
 *         cdef void *buf = rtdef.PyUnicode_DATA(ret)
 */
  __pyx_v_maxchar = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_get_max(__pyx_v_self, __pyx_v_begin, __pyx_v_end);

  /* "_gappedbuf.pyx":196
 *         cdef Py_ssize_t n = end - begin
 *         cdef Py_UCS4 maxchar = self._get_max(begin, end)
 *         cdef unicode ret = rtdef.PyUnicode_New(n, maxchar)             # <<<<<<<<<<<<<<
 *         cdef void *buf = rtdef.PyUnicode_DATA(ret)
 *         cdef nbytes = rtdef.PyUnicode_KIND(ret)
 */
  __pyx_t_8 = PyUnicode_New(__pyx_v_n, __pyx_v_maxchar); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None)||((void)PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_8)->tp_name), 0))) __PYX_ERR(0, 196, __pyx_L1_error)
  __pyx_v_ret = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "_gappedbuf.pyx":197
 *         cdef Py_UCS4 maxchar = self._get_max(begin, end)
 *         cdef unicode ret = rtdef.PyUnicode_New(n, maxchar)
 *         cdef void *buf = rtdef.PyUnicode_DATA(ret)             # <<<<<<<<<<<<<<
 *         cdef nbytes = rtdef.PyUnicode_KIND(ret)
 * 
 */
  __pyx_t_11 = PyUnicode_DATA(__pyx_v_ret); if (unlikely(__pyx_t_11 == ((void *)NULL))) __PYX_ERR(0, 197, __pyx_L1_error)
  __pyx_v_buf = __pyx_t_11;

  /* "_gappedbuf.pyx":198
 *         cdef unicode ret = rtdef.PyUnicode_New(n, maxchar)
 *         cdef void *buf = rtdef.PyUnicode_DATA(ret)
 *         cdef nbytes = rtdef.PyUnicode_KIND(ret)             # <<<<<<<<<<<<<<
 * 
 *         if nbytes == 1:
 */
  __pyx_t_7 = PyUnicode_KIND(__pyx_v_ret); if (unlikely(__pyx_t_7 == ((int)0))) __PYX_ERR(0, 198, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_nbytes = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "_gappedbuf.pyx":200
 *         cdef nbytes = rtdef.PyUnicode_KIND(ret)
 * 
 *         if nbytes == 1:             # <<<<<<<<<<<<<<
 *             self._copy_chars1(buf, begin, end)
 *         elif nbytes == 2:
 */
  __pyx_t_8 = __Pyx_PyInt_EqObjC(__pyx_v_nbytes, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_10) {

    /* "_gappedbuf.pyx":201
 * 
 *         if nbytes == 1:
 *             self._copy_chars1(buf, begin, end)             # <<<<<<<<<<<<<<
 *         elif nbytes == 2:
 *             self._copy_chars2(buf, begin, end)
 */
    ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_copy_chars1(__pyx_v_self, __pyx_v_buf, __pyx_v_begin, __pyx_v_end);

    /* "_gappedbuf.pyx":200
 *         cdef nbytes = rtdef.PyUnicode_KIND(ret)
 * 
 *         if nbytes == 1:             # <<<<<<<<<<<<<<
 *             self._copy_chars1(buf, begin, end)
 *         elif nbytes == 2:
 */
    goto __pyx_L4;
  }

  /* "_gappedbuf.pyx":202
 *         if nbytes == 1:
 *             self._copy_chars1(buf, begin, end)
 *         elif nbytes == 2:             # <<<<<<<<<<<<<<
 *             self._copy_chars2(buf, begin, end)
 *         elif nbytes == 4:
 */
  __pyx_t_8 = __Pyx_PyInt_EqObjC(__pyx_v_nbytes, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_10) {

    /* "_gappedbuf.pyx":203
 *             self._copy_chars1(buf, begin, end)
 *         elif nbytes == 2:
 *             self._copy_chars2(buf, begin, end)             # <<<<<<<<<<<<<<
 *         elif nbytes == 4:
 *             self._copy_chars4(buf, begin, end)
 */
    ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_copy_chars2(__pyx_v_self, __pyx_v_buf, __pyx_v_begin, __pyx_v_end);

    /* "_gappedbuf.pyx":202
 *         if nbytes == 1:
 *             self._copy_chars1(buf, begin, end)
 *         elif nbytes == 2:             # <<<<<<<<<<<<<<
 *             self._copy_chars2(buf, begin, end)
 *         elif nbytes == 4:
 */
    goto __pyx_L4;
  }

  /* "_gappedbuf.pyx":204
 *         elif nbytes == 2:
 *             self._copy_chars2(buf, begin, end)
 *         elif nbytes == 4:             # <<<<<<<<<<<<<<
 *             self._copy_chars4(buf, begin, end)
 *         else:
 */
  __pyx_t_8 = __Pyx_PyInt_EqObjC(__pyx_v_nbytes, __pyx_int_4, 4, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (likely(__pyx_t_10)) {

    /* "_gappedbuf.pyx":205
 *             self._copy_chars2(buf, begin, end)
 *         elif nbytes == 4:
 *             self._copy_chars4(buf, begin, end)             # <<<<<<<<<<<<<<
 *         else:
 *             raise RuntimeError()
 */
    ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_copy_chars4(__pyx_v_self, __pyx_v_buf, __pyx_v_begin, __pyx_v_end);

    /* "_gappedbuf.pyx":204
 *         elif nbytes == 2:
 *             self._copy_chars2(buf, begin, end)
 *         elif nbytes == 4:             # <<<<<<<<<<<<<<
 *             self._copy_chars4(buf, begin, end)
 *         else:
 */
    goto __pyx_L4;
  }

  /* "_gappedbuf.pyx":207
 *             self._copy_chars4(buf, begin, end)
 *         else:
 *             raise RuntimeError()             # <<<<<<<<<<<<<<
 * 
 *         return ret
 */
  /*else*/ {
    __pyx_t_8 = __Pyx_PyObject_CallNoArg(__pyx_builtin_RuntimeError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 207, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "_gappedbuf.pyx":209
 *             raise RuntimeError()
 * 
 *         return ret             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ret);
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":189
 *                     (n-former)*sizeof(Py_UCS4))
 * 
 *     cpdef get(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise IndexError('Invalid range: %d, %d, %d' % (
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_nbytes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_13get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_13get(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get", 1, 2, 2, 1); __PYX_ERR(0, 189, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get") < 0)) __PYX_ERR(0, 189, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 189, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 189, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 189, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_12get(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_begin, __pyx_v_end);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_12get(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_get(__pyx_v_self, __pyx_v_begin, __pyx_v_end, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":211
 *         return ret
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.numelems
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_10_gappedbuf_12GappedBuffer_15__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_10_gappedbuf_12GappedBuffer_15__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_14__len__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_10_gappedbuf_12GappedBuffer_14__len__(struct GappedBufferObj *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "_gappedbuf.pyx":212
 * 
 *     def __len__(self):
 *         return self.numelems             # <<<<<<<<<<<<<<
 * 
 *     cdef int _get_slice(self, object obj,
 */
  __pyx_r = __pyx_v_self->numelems;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":211
 *         return ret
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.numelems
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":214
 *         return self.numelems
 * 
 *     cdef int _get_slice(self, object obj,             # <<<<<<<<<<<<<<
 *             Py_ssize_t *f, Py_ssize_t *t) except *:
 *         cdef Py_ssize_t step, slicelength
 */

static int __pyx_f_10_gappedbuf_12GappedBuffer__get_slice(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_obj, Py_ssize_t *__pyx_v_f, Py_ssize_t *__pyx_v_t) {
  Py_ssize_t __pyx_v_step;
  Py_ssize_t __pyx_v_slicelength;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  long __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_slice", 0);

  /* "_gappedbuf.pyx":218
 *         cdef Py_ssize_t step, slicelength
 * 
 *         if rtdef.PySlice_Check(obj):             # <<<<<<<<<<<<<<
 *             rtdef.PySlice_GetIndicesEx(obj, self.numelems, f, t,
 *                      &step, &slicelength)
 */
  __pyx_t_1 = (PySlice_Check(__pyx_v_obj) != 0);
  if (__pyx_t_1) {

    /* "_gappedbuf.pyx":219
 * 
 *         if rtdef.PySlice_Check(obj):
 *             rtdef.PySlice_GetIndicesEx(obj, self.numelems, f, t,             # <<<<<<<<<<<<<<
 *                      &step, &slicelength)
 *             if step != 1:
 */
    __pyx_t_2 = PySlice_GetIndicesEx(__pyx_v_obj, __pyx_v_self->numelems, __pyx_v_f, __pyx_v_t, (&__pyx_v_step), (&__pyx_v_slicelength)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 219, __pyx_L1_error)

    /* "_gappedbuf.pyx":221
 *             rtdef.PySlice_GetIndicesEx(obj, self.numelems, f, t,
 *                      &step, &slicelength)
 *             if step != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError('step is not supported')
 *             return 1
 */
    __pyx_t_1 = ((__pyx_v_step != 1) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "_gappedbuf.pyx":222
 *                      &step, &slicelength)
 *             if step != 1:
 *                 raise ValueError('step is not supported')             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 222, __pyx_L1_error)

      /* "_gappedbuf.pyx":221
 *             rtdef.PySlice_GetIndicesEx(obj, self.numelems, f, t,
 *                      &step, &slicelength)
 *             if step != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError('step is not supported')
 *             return 1
 */
    }

    /* "_gappedbuf.pyx":223
 *             if step != 1:
 *                 raise ValueError('step is not supported')
 *             return 1             # <<<<<<<<<<<<<<
 *         else:
 *             f[0] = obj
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "_gappedbuf.pyx":218
 *         cdef Py_ssize_t step, slicelength
 * 
 *         if rtdef.PySlice_Check(obj):             # <<<<<<<<<<<<<<
 *             rtdef.PySlice_GetIndicesEx(obj, self.numelems, f, t,
 *                      &step, &slicelength)
 */
  }

  /* "_gappedbuf.pyx":225
 *             return 1
 *         else:
 *             f[0] = obj             # <<<<<<<<<<<<<<
 *             if f[0] < 0:
 *                 f[0] += self.numelems
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyIndex_AsSsize_t(__pyx_v_obj); if (unlikely((__pyx_t_4 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 225, __pyx_L1_error)
    (__pyx_v_f[0]) = __pyx_t_4;

    /* "_gappedbuf.pyx":226
 *         else:
 *             f[0] = obj
 *             if f[0] < 0:             # <<<<<<<<<<<<<<
 *                 f[0] += self.numelems
 *             if not (0 <= f[0] < self.numelems):
 */
    __pyx_t_1 = (((__pyx_v_f[0]) < 0) != 0);
    if (__pyx_t_1) {

      /* "_gappedbuf.pyx":227
 *             f[0] = obj
 *             if f[0] < 0:
 *                 f[0] += self.numelems             # <<<<<<<<<<<<<<
 *             if not (0 <= f[0] < self.numelems):
 *                 raise IndexError('Invalid index value: %d, %d' % (
 */
      __pyx_t_5 = 0;
      (__pyx_v_f[__pyx_t_5]) = ((__pyx_v_f[__pyx_t_5]) + __pyx_v_self->numelems);

      /* "_gappedbuf.pyx":226
 *         else:
 *             f[0] = obj
 *             if f[0] < 0:             # <<<<<<<<<<<<<<
 *                 f[0] += self.numelems
 *             if not (0 <= f[0] < self.numelems):
 */
    }

    /* "_gappedbuf.pyx":228
 *             if f[0] < 0:
 *                 f[0] += self.numelems
 *             if not (0 <= f[0] < self.numelems):             # <<<<<<<<<<<<<<
 *                 raise IndexError('Invalid index value: %d, %d' % (
 *                                     f[0], self.numelems))
 */
    __pyx_t_1 = (0 <= (__pyx_v_f[0]));
    if (__pyx_t_1) {
      __pyx_t_1 = ((__pyx_v_f[0]) < __pyx_v_self->numelems);
    }
    __pyx_t_6 = ((!(__pyx_t_1 != 0)) != 0);
    if (unlikely(__pyx_t_6)) {

      /* "_gappedbuf.pyx":230
 *             if not (0 <= f[0] < self.numelems):
 *                 raise IndexError('Invalid index value: %d, %d' % (
 *                                     f[0], self.numelems))             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
      __pyx_t_3 = PyInt_FromSsize_t((__pyx_v_f[0])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_7);
      __pyx_t_3 = 0;
      __pyx_t_7 = 0;

      /* "_gappedbuf.pyx":229
 *                 f[0] += self.numelems
 *             if not (0 <= f[0] < self.numelems):
 *                 raise IndexError('Invalid index value: %d, %d' % (             # <<<<<<<<<<<<<<
 *                                     f[0], self.numelems))
 *             return 0
 */
      __pyx_t_7 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_index_value_d_d, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 229, __pyx_L1_error)

      /* "_gappedbuf.pyx":228
 *             if f[0] < 0:
 *                 f[0] += self.numelems
 *             if not (0 <= f[0] < self.numelems):             # <<<<<<<<<<<<<<
 *                 raise IndexError('Invalid index value: %d, %d' % (
 *                                     f[0], self.numelems))
 */
    }

    /* "_gappedbuf.pyx":231
 *                 raise IndexError('Invalid index value: %d, %d' % (
 *                                     f[0], self.numelems))
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, object item):
 */
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "_gappedbuf.pyx":214
 *         return self.numelems
 * 
 *     cdef int _get_slice(self, object obj,             # <<<<<<<<<<<<<<
 *             Py_ssize_t *f, Py_ssize_t *t) except *:
 *         cdef Py_ssize_t step, slicelength
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer._get_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":233
 *             return 0
 * 
 *     def __getitem__(self, object item):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t start, stop
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_17__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_17__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_16__getitem__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_16__getitem__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_item) {
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_stop;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "_gappedbuf.pyx":236
 *         cdef Py_ssize_t start, stop
 * 
 *         if self._get_slice(item, &start, &stop):             # <<<<<<<<<<<<<<
 *             return self.get(start, stop)
 *         else:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_get_slice(__pyx_v_self, __pyx_v_item, (&__pyx_v_start), (&__pyx_v_stop)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 236, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "_gappedbuf.pyx":237
 * 
 *         if self._get_slice(item, &start, &stop):
 *             return self.get(start, stop)             # <<<<<<<<<<<<<<
 *         else:
 *             return self.buf[start+(0 if start < self.gap else self.gapsize)]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->get(__pyx_v_self, __pyx_v_start, __pyx_v_stop, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "_gappedbuf.pyx":236
 *         cdef Py_ssize_t start, stop
 * 
 *         if self._get_slice(item, &start, &stop):             # <<<<<<<<<<<<<<
 *             return self.get(start, stop)
 *         else:
 */
  }

  /* "_gappedbuf.pyx":239
 *             return self.get(start, stop)
 *         else:
 *             return self.buf[start+(0 if start < self.gap else self.gapsize)]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, object item, unicode value):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (((__pyx_v_start < __pyx_v_self->gap) != 0)) {
      __pyx_t_4 = 0;
    } else {
      __pyx_t_4 = __pyx_v_self->gapsize;
    }
    __pyx_t_3 = PyUnicode_FromOrdinal((__pyx_v_self->buf[(__pyx_v_start + __pyx_t_4)])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "_gappedbuf.pyx":233
 *             return 0
 * 
 *     def __getitem__(self, object item):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t start, stop
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":241
 *             return self.buf[start+(0 if start < self.gap else self.gapsize)]
 * 
 *     def __setitem__(self, object item, unicode value):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t start, stop
 * 
 */

/* Python wrapper */
static int __pyx_pw_10_gappedbuf_12GappedBuffer_19__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10_gappedbuf_12GappedBuffer_19__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyUnicode_Type), 1, "value", 1))) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_18__setitem__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_item), ((PyObject*)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10_gappedbuf_12GappedBuffer_18__setitem__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_stop;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  Py_UCS4 __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "_gappedbuf.pyx":244
 *         cdef Py_ssize_t start, stop
 * 
 *         if self._get_slice(item, &start, &stop):             # <<<<<<<<<<<<<<
 *             self.replace(start, stop, value)
 *         else:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_get_slice(__pyx_v_self, __pyx_v_item, (&__pyx_v_start), (&__pyx_v_stop)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 244, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "_gappedbuf.pyx":245
 * 
 *         if self._get_slice(item, &start, &stop):
 *             self.replace(start, stop, value)             # <<<<<<<<<<<<<<
 *         else:
 *             if len(value) != 1:
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->replace(__pyx_v_self, __pyx_v_start, __pyx_v_stop, __pyx_v_value, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "_gappedbuf.pyx":244
 *         cdef Py_ssize_t start, stop
 * 
 *         if self._get_slice(item, &start, &stop):             # <<<<<<<<<<<<<<
 *             self.replace(start, stop, value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "_gappedbuf.pyx":247
 *             self.replace(start, stop, value)
 *         else:
 *             if len(value) != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError('Invalid string length: %d' % len(value))
 * 
 */
  /*else*/ {
    if (unlikely(__pyx_v_value == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 247, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_value); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 247, __pyx_L1_error)
    __pyx_t_2 = ((__pyx_t_4 != 1) != 0);
    if (unlikely(__pyx_t_2)) {

      /* "_gappedbuf.pyx":248
 *         else:
 *             if len(value) != 1:
 *                 raise ValueError('Invalid string length: %d' % len(value))             # <<<<<<<<<<<<<<
 * 
 *             start = start+(0 if start < self.gap else self.gapsize)
 */
      if (unlikely(__pyx_v_value == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 248, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_value); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 248, __pyx_L1_error)
      __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_string_length_d, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 248, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 248, __pyx_L1_error)

      /* "_gappedbuf.pyx":247
 *             self.replace(start, stop, value)
 *         else:
 *             if len(value) != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError('Invalid string length: %d' % len(value))
 * 
 */
    }

    /* "_gappedbuf.pyx":250
 *                 raise ValueError('Invalid string length: %d' % len(value))
 * 
 *             start = start+(0 if start < self.gap else self.gapsize)             # <<<<<<<<<<<<<<
 *             self.buf[start] = <Py_UCS4>value
 * 
 */
    if (((__pyx_v_start < __pyx_v_self->gap) != 0)) {
      __pyx_t_4 = 0;
    } else {
      __pyx_t_4 = __pyx_v_self->gapsize;
    }
    __pyx_v_start = (__pyx_v_start + __pyx_t_4);

    /* "_gappedbuf.pyx":251
 * 
 *             start = start+(0 if start < self.gap else self.gapsize)
 *             self.buf[start] = <Py_UCS4>value             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(self, object item):
 */
    __pyx_t_6 = __Pyx_PyObject_AsPy_UCS4(__pyx_v_value); if (unlikely((__pyx_t_6 == (Py_UCS4)-1) && PyErr_Occurred())) __PYX_ERR(0, 251, __pyx_L1_error)
    (__pyx_v_self->buf[__pyx_v_start]) = ((Py_UCS4)__pyx_t_6);
  }
  __pyx_L3:;

  /* "_gappedbuf.pyx":241
 *             return self.buf[start+(0 if start < self.gap else self.gapsize)]
 * 
 *     def __setitem__(self, object item, unicode value):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t start, stop
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":253
 *             self.buf[start] = <Py_UCS4>value
 * 
 *     def __delitem__(self, object item):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t start, stop
 * 
 */

/* Python wrapper */
static int __pyx_pw_10_gappedbuf_12GappedBuffer_21__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static int __pyx_pw_10_gappedbuf_12GappedBuffer_21__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_20__delitem__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10_gappedbuf_12GappedBuffer_20__delitem__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_item) {
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_stop;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__delitem__", 0);

  /* "_gappedbuf.pyx":256
 *         cdef Py_ssize_t start, stop
 * 
 *         if self._get_slice(item, &start, &stop):             # <<<<<<<<<<<<<<
 *             self.delete(start, stop)
 *         else:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_get_slice(__pyx_v_self, __pyx_v_item, (&__pyx_v_start), (&__pyx_v_stop)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "_gappedbuf.pyx":257
 * 
 *         if self._get_slice(item, &start, &stop):
 *             self.delete(start, stop)             # <<<<<<<<<<<<<<
 *         else:
 *             self.delete(start, start+1)
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->delete(__pyx_v_self, __pyx_v_start, __pyx_v_stop, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "_gappedbuf.pyx":256
 *         cdef Py_ssize_t start, stop
 * 
 *         if self._get_slice(item, &start, &stop):             # <<<<<<<<<<<<<<
 *             self.delete(start, stop)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "_gappedbuf.pyx":259
 *             self.delete(start, stop)
 *         else:
 *             self.delete(start, start+1)             # <<<<<<<<<<<<<<
 * 
 *     def findchr(self, unicode c, Py_ssize_t begin, Py_ssize_t end):
 */
  /*else*/ {
    __pyx_t_3 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->delete(__pyx_v_self, __pyx_v_start, (__pyx_v_start + 1), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "_gappedbuf.pyx":253
 *             self.buf[start] = <Py_UCS4>value
 * 
 *     def __delitem__(self, object item):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t start, stop
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":261
 *             self.delete(start, start+1)
 * 
 *     def findchr(self, unicode c, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 *chars
 *         cdef Py_ssize_t size, b, e, p
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_23findchr(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_23findchr(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_c = 0;
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("findchr (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_c,&__pyx_n_s_begin,&__pyx_n_s_end,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_c)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("findchr", 1, 3, 3, 1); __PYX_ERR(0, 261, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("findchr", 1, 3, 3, 2); __PYX_ERR(0, 261, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "findchr") < 0)) __PYX_ERR(0, 261, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_c = ((PyObject*)values[0]);
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 261, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[2]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 261, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("findchr", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 261, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.findchr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c), (&PyUnicode_Type), 1, "c", 1))) __PYX_ERR(0, 261, __pyx_L1_error)
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_22findchr(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_c, __pyx_v_begin, __pyx_v_end);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_22findchr(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_c, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  Py_UCS4 *__pyx_v_chars;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  long __pyx_v_q;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_UCS4 *__pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  char const *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("findchr", 0);

  /* "_gappedbuf.pyx":265
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_1 = (0 <= __pyx_v_begin);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "_gappedbuf.pyx":267
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)
 */
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":266
 * 
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 266, __pyx_L1_error)

    /* "_gappedbuf.pyx":265
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":269
 *                                 begin, end, self.numelems))
 * 
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)             # <<<<<<<<<<<<<<
 *         try:
 *             size = len(c)
 */
  __pyx_t_7 = PyUnicode_AsUCS4Copy(__pyx_v_c); if (unlikely(__pyx_t_7 == ((Py_UCS4 *)NULL))) __PYX_ERR(0, 269, __pyx_L1_error)
  __pyx_v_chars = __pyx_t_7;

  /* "_gappedbuf.pyx":270
 * 
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)
 *         try:             # <<<<<<<<<<<<<<
 *             size = len(c)
 *             # search before gap
 */
  /*try:*/ {

    /* "_gappedbuf.pyx":271
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)
 *         try:
 *             size = len(c)             # <<<<<<<<<<<<<<
 *             # search before gap
 *             e = min(end, self.gap)
 */
    if (unlikely(__pyx_v_c == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 271, __pyx_L5_error)
    }
    __pyx_t_8 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_c); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 271, __pyx_L5_error)
    __pyx_v_size = __pyx_t_8;

    /* "_gappedbuf.pyx":273
 *             size = len(c)
 *             # search before gap
 *             e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *             for p from begin <= p < e:
 *                 for q from 0 <= q < size:
 */
    __pyx_t_8 = __pyx_v_self->gap;
    __pyx_t_9 = __pyx_v_end;
    if (((__pyx_t_8 < __pyx_t_9) != 0)) {
      __pyx_t_10 = __pyx_t_8;
    } else {
      __pyx_t_10 = __pyx_t_9;
    }
    __pyx_v_e = __pyx_t_10;

    /* "_gappedbuf.pyx":274
 *             # search before gap
 *             e = min(end, self.gap)
 *             for p from begin <= p < e:             # <<<<<<<<<<<<<<
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 */
    __pyx_t_10 = __pyx_v_e;
    for (__pyx_v_p = __pyx_v_begin; __pyx_v_p < __pyx_t_10; __pyx_v_p++) {

      /* "_gappedbuf.pyx":275
 *             e = min(end, self.gap)
 *             for p from begin <= p < e:
 *                 for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                     if self.buf[p] == chars[q]:
 *                         return p
 */
      __pyx_t_8 = __pyx_v_size;
      for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_8; __pyx_v_q++) {

        /* "_gappedbuf.pyx":276
 *             for p from begin <= p < e:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p
 * 
 */
        __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_chars[__pyx_v_q])) != 0);
        if (__pyx_t_2) {

          /* "_gappedbuf.pyx":277
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 *                         return p             # <<<<<<<<<<<<<<
 * 
 *             # search after gap
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_p); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 277, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L4_return;

          /* "_gappedbuf.pyx":276
 *             for p from begin <= p < e:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p
 * 
 */
        }
      }
    }

    /* "_gappedbuf.pyx":280
 * 
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *             for p from b <= p < end+self.gapsize:
 *                 for q from 0 <= q < size:
 */
    __pyx_t_10 = __pyx_v_self->gap;
    __pyx_t_8 = __pyx_v_begin;
    if (((__pyx_t_10 > __pyx_t_8) != 0)) {
      __pyx_t_9 = __pyx_t_10;
    } else {
      __pyx_t_9 = __pyx_t_8;
    }
    __pyx_v_b = (__pyx_t_9 + __pyx_v_self->gapsize);

    /* "_gappedbuf.pyx":281
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from b <= p < end+self.gapsize:             # <<<<<<<<<<<<<<
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 */
    __pyx_t_9 = (__pyx_v_end + __pyx_v_self->gapsize);
    for (__pyx_v_p = __pyx_v_b; __pyx_v_p < __pyx_t_9; __pyx_v_p++) {

      /* "_gappedbuf.pyx":282
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from b <= p < end+self.gapsize:
 *                 for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                     if self.buf[p] == chars[q]:
 *                         return p-self.gapsize
 */
      __pyx_t_10 = __pyx_v_size;
      for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_10; __pyx_v_q++) {

        /* "_gappedbuf.pyx":283
 *             for p from b <= p < end+self.gapsize:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p-self.gapsize
 *         finally:
 */
        __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_chars[__pyx_v_q])) != 0);
        if (__pyx_t_2) {

          /* "_gappedbuf.pyx":284
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 *                         return p-self.gapsize             # <<<<<<<<<<<<<<
 *         finally:
 *             mem.PyMem_Free(chars)
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_p - __pyx_v_self->gapsize)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 284, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L4_return;

          /* "_gappedbuf.pyx":283
 *             for p from b <= p < end+self.gapsize:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p-self.gapsize
 *         finally:
 */
        }
      }
    }
  }

  /* "_gappedbuf.pyx":286
 *                         return p-self.gapsize
 *         finally:
 *             mem.PyMem_Free(chars)             # <<<<<<<<<<<<<<
 * 
 *         return -1
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_chars);
      goto __pyx_L6;
    }
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __pyx_t_11 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_13 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_chars);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
      }
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_ErrRestore(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_13;
      goto __pyx_L1_error;
    }
    __pyx_L4_return: {
      __pyx_t_19 = __pyx_r;
      __pyx_r = 0;
      PyMem_Free(__pyx_v_chars);
      __pyx_r = __pyx_t_19;
      __pyx_t_19 = 0;
      goto __pyx_L0;
    }
    __pyx_L6:;
  }

  /* "_gappedbuf.pyx":288
 *             mem.PyMem_Free(chars)
 * 
 *         return -1             # <<<<<<<<<<<<<<
 * 
 *     def rfindchr(self, unicode c, Py_ssize_t begin, Py_ssize_t end):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_neg_1);
  __pyx_r = __pyx_int_neg_1;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":261
 *             self.delete(start, start+1)
 * 
 *     def findchr(self, unicode c, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 *chars
 *         cdef Py_ssize_t size, b, e, p
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.findchr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":290
 *         return -1
 * 
 *     def rfindchr(self, unicode c, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 *chars
 *         cdef Py_ssize_t size, b, e, p
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_25rfindchr(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_25rfindchr(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_c = 0;
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rfindchr (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_c,&__pyx_n_s_begin,&__pyx_n_s_end,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_c)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rfindchr", 1, 3, 3, 1); __PYX_ERR(0, 290, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rfindchr", 1, 3, 3, 2); __PYX_ERR(0, 290, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "rfindchr") < 0)) __PYX_ERR(0, 290, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_c = ((PyObject*)values[0]);
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 290, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[2]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 290, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("rfindchr", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 290, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.rfindchr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c), (&PyUnicode_Type), 1, "c", 1))) __PYX_ERR(0, 290, __pyx_L1_error)
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_24rfindchr(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_c, __pyx_v_begin, __pyx_v_end);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_24rfindchr(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_c, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  Py_UCS4 *__pyx_v_chars;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  long __pyx_v_q;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_UCS4 *__pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  char const *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("rfindchr", 0);

  /* "_gappedbuf.pyx":294
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_1 = (0 <= __pyx_v_begin);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "_gappedbuf.pyx":296
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)
 */
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":295
 * 
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 295, __pyx_L1_error)

    /* "_gappedbuf.pyx":294
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":298
 *                                 begin, end, self.numelems))
 * 
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)             # <<<<<<<<<<<<<<
 *         try:
 *             size = len(c)
 */
  __pyx_t_7 = PyUnicode_AsUCS4Copy(__pyx_v_c); if (unlikely(__pyx_t_7 == ((Py_UCS4 *)NULL))) __PYX_ERR(0, 298, __pyx_L1_error)
  __pyx_v_chars = __pyx_t_7;

  /* "_gappedbuf.pyx":299
 * 
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)
 *         try:             # <<<<<<<<<<<<<<
 *             size = len(c)
 *             # search after gap
 */
  /*try:*/ {

    /* "_gappedbuf.pyx":300
 *         chars = rtdef.PyUnicode_AsUCS4Copy(c)
 *         try:
 *             size = len(c)             # <<<<<<<<<<<<<<
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize
 */
    if (unlikely(__pyx_v_c == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 300, __pyx_L5_error)
    }
    __pyx_t_8 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_c); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 300, __pyx_L5_error)
    __pyx_v_size = __pyx_t_8;

    /* "_gappedbuf.pyx":302
 *             size = len(c)
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *             for p from (end+self.gapsize) > p >= b:
 *                 for q from 0 <= q < size:
 */
    __pyx_t_8 = __pyx_v_self->gap;
    __pyx_t_9 = __pyx_v_begin;
    if (((__pyx_t_8 > __pyx_t_9) != 0)) {
      __pyx_t_10 = __pyx_t_8;
    } else {
      __pyx_t_10 = __pyx_t_9;
    }
    __pyx_v_b = (__pyx_t_10 + __pyx_v_self->gapsize);

    /* "_gappedbuf.pyx":303
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from (end+self.gapsize) > p >= b:             # <<<<<<<<<<<<<<
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 */
    __pyx_t_10 = __pyx_v_b;
    for (__pyx_v_p = (__pyx_v_end + __pyx_v_self->gapsize)-1; __pyx_v_p >= __pyx_t_10; __pyx_v_p--) {

      /* "_gappedbuf.pyx":304
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from (end+self.gapsize) > p >= b:
 *                 for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                     if self.buf[p] == chars[q]:
 *                         return p-self.gapsize
 */
      __pyx_t_8 = __pyx_v_size;
      for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_8; __pyx_v_q++) {

        /* "_gappedbuf.pyx":305
 *             for p from (end+self.gapsize) > p >= b:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p-self.gapsize
 * 
 */
        __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_chars[__pyx_v_q])) != 0);
        if (__pyx_t_2) {

          /* "_gappedbuf.pyx":306
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 *                         return p-self.gapsize             # <<<<<<<<<<<<<<
 * 
 *             # search before gap
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_p - __pyx_v_self->gapsize)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 306, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L4_return;

          /* "_gappedbuf.pyx":305
 *             for p from (end+self.gapsize) > p >= b:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p-self.gapsize
 * 
 */
        }
      }
    }

    /* "_gappedbuf.pyx":309
 * 
 *             # search before gap
 *             e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *             for p from e > p >= begin:
 *                 for q from 0 <= q < size:
 */
    __pyx_t_10 = __pyx_v_self->gap;
    __pyx_t_8 = __pyx_v_end;
    if (((__pyx_t_10 < __pyx_t_8) != 0)) {
      __pyx_t_9 = __pyx_t_10;
    } else {
      __pyx_t_9 = __pyx_t_8;
    }
    __pyx_v_e = __pyx_t_9;

    /* "_gappedbuf.pyx":310
 *             # search before gap
 *             e = min(end, self.gap)
 *             for p from e > p >= begin:             # <<<<<<<<<<<<<<
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 */
    __pyx_t_9 = __pyx_v_begin;
    for (__pyx_v_p = __pyx_v_e-1; __pyx_v_p >= __pyx_t_9; __pyx_v_p--) {

      /* "_gappedbuf.pyx":311
 *             e = min(end, self.gap)
 *             for p from e > p >= begin:
 *                 for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                     if self.buf[p] == chars[q]:
 *                         return p
 */
      __pyx_t_10 = __pyx_v_size;
      for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_10; __pyx_v_q++) {

        /* "_gappedbuf.pyx":312
 *             for p from e > p >= begin:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p
 *         finally:
 */
        __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_chars[__pyx_v_q])) != 0);
        if (__pyx_t_2) {

          /* "_gappedbuf.pyx":313
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:
 *                         return p             # <<<<<<<<<<<<<<
 *         finally:
 *             mem.PyMem_Free(chars)
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_p); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 313, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L4_return;

          /* "_gappedbuf.pyx":312
 *             for p from e > p >= begin:
 *                 for q from 0 <= q < size:
 *                     if self.buf[p] == chars[q]:             # <<<<<<<<<<<<<<
 *                         return p
 *         finally:
 */
        }
      }
    }
  }

  /* "_gappedbuf.pyx":315
 *                         return p
 *         finally:
 *             mem.PyMem_Free(chars)             # <<<<<<<<<<<<<<
 * 
 *         return -1
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_chars);
      goto __pyx_L6;
    }
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __pyx_t_11 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_13 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_chars);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
      }
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_ErrRestore(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_13;
      goto __pyx_L1_error;
    }
    __pyx_L4_return: {
      __pyx_t_19 = __pyx_r;
      __pyx_r = 0;
      PyMem_Free(__pyx_v_chars);
      __pyx_r = __pyx_t_19;
      __pyx_t_19 = 0;
      goto __pyx_L0;
    }
    __pyx_L6:;
  }

  /* "_gappedbuf.pyx":317
 *             mem.PyMem_Free(chars)
 * 
 *         return -1             # <<<<<<<<<<<<<<
 * 
 *     cpdef getint(self, Py_ssize_t pos):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_neg_1);
  __pyx_r = __pyx_int_neg_1;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":290
 *         return -1
 * 
 *     def rfindchr(self, unicode c, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 *         cdef Py_UCS4 *chars
 *         cdef Py_ssize_t size, b, e, p
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.rfindchr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":319
 *         return -1
 * 
 *     cpdef getint(self, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *         if not (0 <= pos < self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_27getint(PyObject *__pyx_v_self, PyObject *__pyx_arg_pos); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_getint(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_pos, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getint", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getint); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_27getint)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":320
 * 
 *     cpdef getint(self, Py_ssize_t pos):
 *         if not (0 <= pos < self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 pos, self.numelems))
 */
  __pyx_t_6 = (0 <= __pyx_v_pos);
  if (__pyx_t_6) {
    __pyx_t_6 = (__pyx_v_pos < __pyx_v_self->numelems);
  }
  __pyx_t_7 = ((!(__pyx_t_6 != 0)) != 0);
  if (unlikely(__pyx_t_7)) {

    /* "_gappedbuf.pyx":322
 *         if not (0 <= pos < self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 pos, self.numelems))             # <<<<<<<<<<<<<<
 *         if pos < self.gap:
 *             return <unsigned long>self.buf[pos]
 */
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;

    /* "_gappedbuf.pyx":321
 *     cpdef getint(self, Py_ssize_t pos):
 *         if not (0 <= pos < self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 pos, self.numelems))
 *         if pos < self.gap:
 */
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_index_value_d_d, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 321, __pyx_L1_error)

    /* "_gappedbuf.pyx":320
 * 
 *     cpdef getint(self, Py_ssize_t pos):
 *         if not (0 <= pos < self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 pos, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":323
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 pos, self.numelems))
 *         if pos < self.gap:             # <<<<<<<<<<<<<<
 *             return <unsigned long>self.buf[pos]
 *         else:
 */
  __pyx_t_7 = ((__pyx_v_pos < __pyx_v_self->gap) != 0);
  if (__pyx_t_7) {

    /* "_gappedbuf.pyx":324
 *                                 pos, self.numelems))
 *         if pos < self.gap:
 *             return <unsigned long>self.buf[pos]             # <<<<<<<<<<<<<<
 *         else:
 *             return <unsigned long>self.buf[self.gapsize+pos]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_unsigned_long(((unsigned long)(__pyx_v_self->buf[__pyx_v_pos]))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "_gappedbuf.pyx":323
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 pos, self.numelems))
 *         if pos < self.gap:             # <<<<<<<<<<<<<<
 *             return <unsigned long>self.buf[pos]
 *         else:
 */
  }

  /* "_gappedbuf.pyx":326
 *             return <unsigned long>self.buf[pos]
 *         else:
 *             return <unsigned long>self.buf[self.gapsize+pos]             # <<<<<<<<<<<<<<
 * 
 *     cpdef getints(self, Py_ssize_t begin, Py_ssize_t end):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_unsigned_long(((unsigned long)(__pyx_v_self->buf[(__pyx_v_self->gapsize + __pyx_v_pos)]))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "_gappedbuf.pyx":319
 *         return -1
 * 
 *     cpdef getint(self, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *         if not (0 <= pos < self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.getint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_27getint(PyObject *__pyx_v_self, PyObject *__pyx_arg_pos); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_27getint(PyObject *__pyx_v_self, PyObject *__pyx_arg_pos) {
  Py_ssize_t __pyx_v_pos;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getint (wrapper)", 0);
  assert(__pyx_arg_pos); {
    __pyx_v_pos = __Pyx_PyIndex_AsSsize_t(__pyx_arg_pos); if (unlikely((__pyx_v_pos == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 319, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.getint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_26getint(((struct GappedBufferObj *)__pyx_v_self), ((Py_ssize_t)__pyx_v_pos));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_26getint(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getint", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_getint(__pyx_v_self, __pyx_v_pos, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.getint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":328
 *             return <unsigned long>self.buf[self.gapsize+pos]
 * 
 *     cpdef getints(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 * 
 *         cdef Py_ssize_t b, e, p, n
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_29getints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_getints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_skip_dispatch) {
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  Py_ssize_t __pyx_v_n;
  PyObject *__pyx_v_ret = 0;
  PyObject *__pyx_v_v = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getints", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getints); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 328, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_29getints)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 328, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 328, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":334
 *         cdef object v
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_9 = (0 <= __pyx_v_begin);
  if (__pyx_t_9) {
    __pyx_t_9 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_9) {
      __pyx_t_9 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_10 = ((!(__pyx_t_9 != 0)) != 0);
  if (unlikely(__pyx_t_10)) {

    /* "_gappedbuf.pyx":336
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         ret = PyList_New(end-begin)
 */
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_5);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":335
 * 
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 335, __pyx_L1_error)

    /* "_gappedbuf.pyx":334
 *         cdef object v
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":338
 *                                 begin, end, self.numelems))
 * 
 *         ret = PyList_New(end-begin)             # <<<<<<<<<<<<<<
 *         n = 0
 *         # read before gap
 */
  __pyx_t_8 = PyList_New((__pyx_v_end - __pyx_v_begin)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 338, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_ret = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "_gappedbuf.pyx":339
 * 
 *         ret = PyList_New(end-begin)
 *         n = 0             # <<<<<<<<<<<<<<
 *         # read before gap
 *         e = min(end, self.gap)
 */
  __pyx_v_n = 0;

  /* "_gappedbuf.pyx":341
 *         n = 0
 *         # read before gap
 *         e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *         for p from begin <= p < e:
 *             v = <unsigned long>self.buf[p]
 */
  __pyx_t_11 = __pyx_v_self->gap;
  __pyx_t_12 = __pyx_v_end;
  if (((__pyx_t_11 < __pyx_t_12) != 0)) {
    __pyx_t_13 = __pyx_t_11;
  } else {
    __pyx_t_13 = __pyx_t_12;
  }
  __pyx_v_e = __pyx_t_13;

  /* "_gappedbuf.pyx":342
 *         # read before gap
 *         e = min(end, self.gap)
 *         for p from begin <= p < e:             # <<<<<<<<<<<<<<
 *             v = <unsigned long>self.buf[p]
 *             PyList_SET_ITEM(ret, n, v)
 */
  __pyx_t_13 = __pyx_v_e;
  for (__pyx_v_p = __pyx_v_begin; __pyx_v_p < __pyx_t_13; __pyx_v_p++) {

    /* "_gappedbuf.pyx":343
 *         e = min(end, self.gap)
 *         for p from begin <= p < e:
 *             v = <unsigned long>self.buf[p]             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(ret, n, v)
 *             ref.Py_INCREF(v)
 */
    __pyx_t_8 = __Pyx_PyInt_From_unsigned_long(((unsigned long)(__pyx_v_self->buf[__pyx_v_p]))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "_gappedbuf.pyx":344
 *         for p from begin <= p < e:
 *             v = <unsigned long>self.buf[p]
 *             PyList_SET_ITEM(ret, n, v)             # <<<<<<<<<<<<<<
 *             ref.Py_INCREF(v)
 *             n += 1
 */
    PyList_SET_ITEM(__pyx_v_ret, __pyx_v_n, __pyx_v_v);

    /* "_gappedbuf.pyx":345
 *             v = <unsigned long>self.buf[p]
 *             PyList_SET_ITEM(ret, n, v)
 *             ref.Py_INCREF(v)             # <<<<<<<<<<<<<<
 *             n += 1
 * 
 */
    Py_INCREF(__pyx_v_v);

    /* "_gappedbuf.pyx":346
 *             PyList_SET_ITEM(ret, n, v)
 *             ref.Py_INCREF(v)
 *             n += 1             # <<<<<<<<<<<<<<
 * 
 *         # read after gap
 */
    __pyx_v_n = (__pyx_v_n + 1);
  }

  /* "_gappedbuf.pyx":349
 * 
 *         # read after gap
 *         b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *         for p from b <= p < end+self.gapsize:
 *             v = <unsigned long>self.buf[p]
 */
  __pyx_t_13 = __pyx_v_self->gap;
  __pyx_t_11 = __pyx_v_begin;
  if (((__pyx_t_13 > __pyx_t_11) != 0)) {
    __pyx_t_12 = __pyx_t_13;
  } else {
    __pyx_t_12 = __pyx_t_11;
  }
  __pyx_v_b = (__pyx_t_12 + __pyx_v_self->gapsize);

  /* "_gappedbuf.pyx":350
 *         # read after gap
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:             # <<<<<<<<<<<<<<
 *             v = <unsigned long>self.buf[p]
 *             PyList_SET_ITEM(ret, n, v)
 */
  __pyx_t_12 = (__pyx_v_end + __pyx_v_self->gapsize);
  for (__pyx_v_p = __pyx_v_b; __pyx_v_p < __pyx_t_12; __pyx_v_p++) {

    /* "_gappedbuf.pyx":351
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:
 *             v = <unsigned long>self.buf[p]             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(ret, n, v)
 *             ref.Py_INCREF(v)
 */
    __pyx_t_8 = __Pyx_PyInt_From_unsigned_long(((unsigned long)(__pyx_v_self->buf[__pyx_v_p]))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "_gappedbuf.pyx":352
 *         for p from b <= p < end+self.gapsize:
 *             v = <unsigned long>self.buf[p]
 *             PyList_SET_ITEM(ret, n, v)             # <<<<<<<<<<<<<<
 *             ref.Py_INCREF(v)
 *             n += 1
 */
    PyList_SET_ITEM(__pyx_v_ret, __pyx_v_n, __pyx_v_v);

    /* "_gappedbuf.pyx":353
 *             v = <unsigned long>self.buf[p]
 *             PyList_SET_ITEM(ret, n, v)
 *             ref.Py_INCREF(v)             # <<<<<<<<<<<<<<
 *             n += 1
 * 
 */
    Py_INCREF(__pyx_v_v);

    /* "_gappedbuf.pyx":354
 *             PyList_SET_ITEM(ret, n, v)
 *             ref.Py_INCREF(v)
 *             n += 1             # <<<<<<<<<<<<<<
 * 
 *         return ret
 */
    __pyx_v_n = (__pyx_v_n + 1);
  }

  /* "_gappedbuf.pyx":356
 *             n += 1
 * 
 *         return ret             # <<<<<<<<<<<<<<
 * 
 *     cdef _insertints(self, Py_ssize_t index, object s):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ret);
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":328
 *             return <unsigned long>self.buf[self.gapsize+pos]
 * 
 *     cpdef getints(self, Py_ssize_t begin, Py_ssize_t end):             # <<<<<<<<<<<<<<
 * 
 *         cdef Py_ssize_t b, e, p, n
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.getints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_29getints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_29getints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getints (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("getints", 1, 2, 2, 1); __PYX_ERR(0, 328, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "getints") < 0)) __PYX_ERR(0, 328, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 328, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 328, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("getints", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 328, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.getints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_28getints(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_begin, __pyx_v_end);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_28getints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getints", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_getints(__pyx_v_self, __pyx_v_begin, __pyx_v_end, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.getints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":358
 *         return ret
 * 
 *     cdef _insertints(self, Py_ssize_t index, object s):             # <<<<<<<<<<<<<<
 * 
 *         cdef Py_ssize_t size = len(s)
 */

static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer__insertints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s) {
  Py_ssize_t __pyx_v_size;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  long __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  unsigned long __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_insertints", 0);

  /* "_gappedbuf.pyx":360
 *     cdef _insertints(self, Py_ssize_t index, object s):
 * 
 *         cdef Py_ssize_t size = len(s)             # <<<<<<<<<<<<<<
 *         if not (0 <= index <= self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_s); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 360, __pyx_L1_error)
  __pyx_v_size = __pyx_t_1;

  /* "_gappedbuf.pyx":361
 * 
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 index, self.numelems))
 */
  __pyx_t_2 = (0 <= __pyx_v_index);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_index <= __pyx_v_self->numelems);
  }
  __pyx_t_3 = ((!(__pyx_t_2 != 0)) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "_gappedbuf.pyx":363
 *         if not (0 <= index <= self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 index, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         if size:
 */
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":362
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):
 *             raise ValueError('Invalid index value: %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 index, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_index_value_d_d, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 362, __pyx_L1_error)

    /* "_gappedbuf.pyx":361
 * 
 *         cdef Py_ssize_t size = len(s)
 *         if not (0 <= index <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid index value: %d, %d' % (
 *                                 index, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":365
 *                                 index, self.numelems))
 * 
 *         if size:             # <<<<<<<<<<<<<<
 *             self._move_gap(index)
 *             if self.gapsize < size:
 */
  __pyx_t_3 = (__pyx_v_size != 0);
  if (__pyx_t_3) {

    /* "_gappedbuf.pyx":366
 * 
 *         if size:
 *             self._move_gap(index)             # <<<<<<<<<<<<<<
 *             if self.gapsize < size:
 *                 self._expand_gap(max(1024, size*2))
 */
    ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_move_gap(__pyx_v_self, __pyx_v_index);

    /* "_gappedbuf.pyx":367
 *         if size:
 *             self._move_gap(index)
 *             if self.gapsize < size:             # <<<<<<<<<<<<<<
 *                 self._expand_gap(max(1024, size*2))
 * 
 */
    __pyx_t_3 = ((__pyx_v_self->gapsize < __pyx_v_size) != 0);
    if (__pyx_t_3) {

      /* "_gappedbuf.pyx":368
 *             self._move_gap(index)
 *             if self.gapsize < size:
 *                 self._expand_gap(max(1024, size*2))             # <<<<<<<<<<<<<<
 * 
 *             for value in s:
 */
      __pyx_t_1 = (__pyx_v_size * 2);
      __pyx_t_7 = 0x400;
      if (((__pyx_t_1 > __pyx_t_7) != 0)) {
        __pyx_t_8 = __pyx_t_1;
      } else {
        __pyx_t_8 = __pyx_t_7;
      }
      ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_expand_gap(__pyx_v_self, __pyx_t_8);

      /* "_gappedbuf.pyx":367
 *         if size:
 *             self._move_gap(index)
 *             if self.gapsize < size:             # <<<<<<<<<<<<<<
 *                 self._expand_gap(max(1024, size*2))
 * 
 */
    }

    /* "_gappedbuf.pyx":370
 *                 self._expand_gap(max(1024, size*2))
 * 
 *             for value in s:             # <<<<<<<<<<<<<<
 *                 self.buf[self.gap] = <unsigned long>value
 *                 self.gap += 1
 */
    if (likely(PyList_CheckExact(__pyx_v_s)) || PyTuple_CheckExact(__pyx_v_s)) {
      __pyx_t_6 = __pyx_v_s; __Pyx_INCREF(__pyx_t_6); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_v_s); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 370, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = Py_TYPE(__pyx_t_6)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 370, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_6))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_6)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_8); __Pyx_INCREF(__pyx_t_5); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 370, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_6, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 370, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_6)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_8); __Pyx_INCREF(__pyx_t_5); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 370, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_6, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 370, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_9(__pyx_t_6);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 370, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "_gappedbuf.pyx":371
 * 
 *             for value in s:
 *                 self.buf[self.gap] = <unsigned long>value             # <<<<<<<<<<<<<<
 *                 self.gap += 1
 * 
 */
      __pyx_t_10 = __Pyx_PyInt_As_unsigned_long(__pyx_v_value); if (unlikely((__pyx_t_10 == (unsigned long)-1) && PyErr_Occurred())) __PYX_ERR(0, 371, __pyx_L1_error)
      (__pyx_v_self->buf[__pyx_v_self->gap]) = ((unsigned long)__pyx_t_10);

      /* "_gappedbuf.pyx":372
 *             for value in s:
 *                 self.buf[self.gap] = <unsigned long>value
 *                 self.gap += 1             # <<<<<<<<<<<<<<
 * 
 *             self.numelems += size
 */
      __pyx_v_self->gap = (__pyx_v_self->gap + 1);

      /* "_gappedbuf.pyx":370
 *                 self._expand_gap(max(1024, size*2))
 * 
 *             for value in s:             # <<<<<<<<<<<<<<
 *                 self.buf[self.gap] = <unsigned long>value
 *                 self.gap += 1
 */
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "_gappedbuf.pyx":374
 *                 self.gap += 1
 * 
 *             self.numelems += size             # <<<<<<<<<<<<<<
 *             self.gapsize -= size
 * 
 */
    __pyx_v_self->numelems = (__pyx_v_self->numelems + __pyx_v_size);

    /* "_gappedbuf.pyx":375
 * 
 *             self.numelems += size
 *             self.gapsize -= size             # <<<<<<<<<<<<<<
 * 
 *     cpdef insertints(self, Py_ssize_t index, object s):
 */
    __pyx_v_self->gapsize = (__pyx_v_self->gapsize - __pyx_v_size);

    /* "_gappedbuf.pyx":365
 *                                 index, self.numelems))
 * 
 *         if size:             # <<<<<<<<<<<<<<
 *             self._move_gap(index)
 *             if self.gapsize < size:
 */
  }

  /* "_gappedbuf.pyx":358
 *         return ret
 * 
 *     cdef _insertints(self, Py_ssize_t index, object s):             # <<<<<<<<<<<<<<
 * 
 *         cdef Py_ssize_t size = len(s)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer._insertints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":377
 *             self.gapsize -= size
 * 
 *     cpdef insertints(self, Py_ssize_t index, object s):             # <<<<<<<<<<<<<<
 * 
 *         self._insertints(index, s)
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_31insertints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_insertints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insertints", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_insertints); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_31insertints)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 377, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 377, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
          __Pyx_INCREF(__pyx_v_s);
          __Pyx_GIVEREF(__pyx_v_s);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_s);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":379
 *     cpdef insertints(self, Py_ssize_t index, object s):
 * 
 *         self._insertints(index, s)             # <<<<<<<<<<<<<<
 * 
 *     cpdef appendints(self, object s):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_insertints(__pyx_v_self, __pyx_v_index, __pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":377
 *             self.gapsize -= size
 * 
 *     cpdef insertints(self, Py_ssize_t index, object s):             # <<<<<<<<<<<<<<
 * 
 *         self._insertints(index, s)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.insertints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_31insertints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_31insertints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_index;
  PyObject *__pyx_v_s = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insertints (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_s,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_s)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("insertints", 1, 2, 2, 1); __PYX_ERR(0, 377, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "insertints") < 0)) __PYX_ERR(0, 377, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_index == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 377, __pyx_L3_error)
    __pyx_v_s = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("insertints", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 377, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.insertints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_30insertints(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_index, __pyx_v_s);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_30insertints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_index, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insertints", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_insertints(__pyx_v_self, __pyx_v_index, __pyx_v_s, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.insertints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":381
 *         self._insertints(index, s)
 * 
 *     cpdef appendints(self, object s):             # <<<<<<<<<<<<<<
 *         self.insertints(self.numelems, s)
 * 
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_33appendints(PyObject *__pyx_v_self, PyObject *__pyx_v_s); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_appendints(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("appendints", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_appendints); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_33appendints)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_s) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_s);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 381, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":382
 * 
 *     cpdef appendints(self, object s):
 *         self.insertints(self.numelems, s)             # <<<<<<<<<<<<<<
 * 
 *     cpdef replaceints(self, Py_ssize_t begin, Py_ssize_t end, object s):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->insertints(__pyx_v_self, __pyx_v_self->numelems, __pyx_v_s, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":381
 *         self._insertints(index, s)
 * 
 *     cpdef appendints(self, object s):             # <<<<<<<<<<<<<<
 *         self.insertints(self.numelems, s)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.appendints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_33appendints(PyObject *__pyx_v_self, PyObject *__pyx_v_s); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_33appendints(PyObject *__pyx_v_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("appendints (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_32appendints(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_s));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_32appendints(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("appendints", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_appendints(__pyx_v_self, __pyx_v_s, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.appendints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":384
 *         self.insertints(self.numelems, s)
 * 
 *     cpdef replaceints(self, Py_ssize_t begin, Py_ssize_t end, object s):             # <<<<<<<<<<<<<<
 *         self._delete(begin, end)
 *         self._insertints(begin, s)
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_35replaceints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_replaceints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("replaceints", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_replaceints); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_35replaceints)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 384, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 384, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 384, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_v_s};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 384, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 384, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __Pyx_INCREF(__pyx_v_s);
          __Pyx_GIVEREF(__pyx_v_s);
          PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_v_s);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 384, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":385
 * 
 *     cpdef replaceints(self, Py_ssize_t begin, Py_ssize_t end, object s):
 *         self._delete(begin, end)             # <<<<<<<<<<<<<<
 *         self._insertints(begin, s)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_delete(__pyx_v_self, __pyx_v_begin, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":386
 *     cpdef replaceints(self, Py_ssize_t begin, Py_ssize_t end, object s):
 *         self._delete(begin, end)
 *         self._insertints(begin, s)             # <<<<<<<<<<<<<<
 * 
 *     cpdef setints(self, Py_ssize_t begin, Py_ssize_t end, unsigned long v):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer *)__pyx_v_self->__pyx_vtab)->_insertints(__pyx_v_self, __pyx_v_begin, __pyx_v_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":384
 *         self.insertints(self.numelems, s)
 * 
 *     cpdef replaceints(self, Py_ssize_t begin, Py_ssize_t end, object s):             # <<<<<<<<<<<<<<
 *         self._delete(begin, end)
 *         self._insertints(begin, s)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.replaceints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_35replaceints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_35replaceints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_s = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("replaceints (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,&__pyx_n_s_s,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("replaceints", 1, 3, 3, 1); __PYX_ERR(0, 384, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_s)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("replaceints", 1, 3, 3, 2); __PYX_ERR(0, 384, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "replaceints") < 0)) __PYX_ERR(0, 384, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 384, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 384, __pyx_L3_error)
    __pyx_v_s = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("replaceints", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 384, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.replaceints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_34replaceints(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_begin, __pyx_v_end, __pyx_v_s);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_34replaceints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("replaceints", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_replaceints(__pyx_v_self, __pyx_v_begin, __pyx_v_end, __pyx_v_s, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.replaceints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":388
 *         self._insertints(begin, s)
 * 
 *     cpdef setints(self, Py_ssize_t begin, Py_ssize_t end, unsigned long v):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t b, e, p
 * 
 */

static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_37setints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_10_gappedbuf_12GappedBuffer_setints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, unsigned long __pyx_v_v, int __pyx_skip_dispatch) {
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  CYTHON_UNUSED long __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setints", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setints); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 388, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_10_gappedbuf_12GappedBuffer_37setints)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 388, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 388, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyInt_From_unsigned_long(__pyx_v_v); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 388, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_6 = __pyx_t_1; __pyx_t_7 = NULL;
        __pyx_t_8 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_8 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 388, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_5 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "_gappedbuf.pyx":391
 *         cdef Py_ssize_t b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_10 = (0 <= __pyx_v_begin);
  if (__pyx_t_10) {
    __pyx_t_10 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_10) {
      __pyx_t_10 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
  if (unlikely(__pyx_t_11)) {

    /* "_gappedbuf.pyx":393
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         n = 0
 */
    __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_6);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_t_6 = 0;

    /* "_gappedbuf.pyx":392
 * 
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __PYX_ERR(0, 392, __pyx_L1_error)

    /* "_gappedbuf.pyx":391
 *         cdef Py_ssize_t b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":395
 *                                 begin, end, self.numelems))
 * 
 *         n = 0             # <<<<<<<<<<<<<<
 *         # replace before gap
 *         e = min(end, self.gap)
 */
  __pyx_v_n = 0;

  /* "_gappedbuf.pyx":397
 *         n = 0
 *         # replace before gap
 *         e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *         for p from begin <= p < e:
 *             self.buf[p] = v
 */
  __pyx_t_12 = __pyx_v_self->gap;
  __pyx_t_13 = __pyx_v_end;
  if (((__pyx_t_12 < __pyx_t_13) != 0)) {
    __pyx_t_14 = __pyx_t_12;
  } else {
    __pyx_t_14 = __pyx_t_13;
  }
  __pyx_v_e = __pyx_t_14;

  /* "_gappedbuf.pyx":398
 *         # replace before gap
 *         e = min(end, self.gap)
 *         for p from begin <= p < e:             # <<<<<<<<<<<<<<
 *             self.buf[p] = v
 * 
 */
  __pyx_t_14 = __pyx_v_e;
  for (__pyx_v_p = __pyx_v_begin; __pyx_v_p < __pyx_t_14; __pyx_v_p++) {

    /* "_gappedbuf.pyx":399
 *         e = min(end, self.gap)
 *         for p from begin <= p < e:
 *             self.buf[p] = v             # <<<<<<<<<<<<<<
 * 
 *         # replace after gap
 */
    (__pyx_v_self->buf[__pyx_v_p]) = __pyx_v_v;
  }

  /* "_gappedbuf.pyx":402
 * 
 *         # replace after gap
 *         b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *         for p from b <= p < end+self.gapsize:
 *             self.buf[p] = v
 */
  __pyx_t_14 = __pyx_v_self->gap;
  __pyx_t_12 = __pyx_v_begin;
  if (((__pyx_t_14 > __pyx_t_12) != 0)) {
    __pyx_t_13 = __pyx_t_14;
  } else {
    __pyx_t_13 = __pyx_t_12;
  }
  __pyx_v_b = (__pyx_t_13 + __pyx_v_self->gapsize);

  /* "_gappedbuf.pyx":403
 *         # replace after gap
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:             # <<<<<<<<<<<<<<
 *             self.buf[p] = v
 * 
 */
  __pyx_t_13 = (__pyx_v_end + __pyx_v_self->gapsize);
  for (__pyx_v_p = __pyx_v_b; __pyx_v_p < __pyx_t_13; __pyx_v_p++) {

    /* "_gappedbuf.pyx":404
 *         b = max(begin, self.gap) + self.gapsize
 *         for p from b <= p < end+self.gapsize:
 *             self.buf[p] = v             # <<<<<<<<<<<<<<
 * 
 *     def findint(self, object nums, Py_ssize_t begin, Py_ssize_t end,
 */
    (__pyx_v_self->buf[__pyx_v_p]) = __pyx_v_v;
  }

  /* "_gappedbuf.pyx":388
 *         self._insertints(begin, s)
 * 
 *     cpdef setints(self, Py_ssize_t begin, Py_ssize_t end, unsigned long v):             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t b, e, p
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.setints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_37setints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_37setints(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  unsigned long __pyx_v_v;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setints (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,&__pyx_n_s_v,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("setints", 1, 3, 3, 1); __PYX_ERR(0, 388, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("setints", 1, 3, 3, 2); __PYX_ERR(0, 388, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "setints") < 0)) __PYX_ERR(0, 388, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[0]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 388, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 388, __pyx_L3_error)
    __pyx_v_v = __Pyx_PyInt_As_unsigned_long(values[2]); if (unlikely((__pyx_v_v == (unsigned long)-1) && PyErr_Occurred())) __PYX_ERR(0, 388, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setints", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 388, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.setints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_36setints(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_begin, __pyx_v_end, __pyx_v_v);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_36setints(struct GappedBufferObj *__pyx_v_self, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, unsigned long __pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setints", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10_gappedbuf_12GappedBuffer_setints(__pyx_v_self, __pyx_v_begin, __pyx_v_end, __pyx_v_v, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.setints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":406
 *             self.buf[p] = v
 * 
 *     def findint(self, object nums, Py_ssize_t begin, Py_ssize_t end,             # <<<<<<<<<<<<<<
 *                 int comp_ne):
 *         cdef unsigned long *longs
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_39findint(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_39findint(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_nums = 0;
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  int __pyx_v_comp_ne;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("findint (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nums,&__pyx_n_s_begin,&__pyx_n_s_end,&__pyx_n_s_comp_ne,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nums)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("findint", 1, 4, 4, 1); __PYX_ERR(0, 406, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("findint", 1, 4, 4, 2); __PYX_ERR(0, 406, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_comp_ne)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("findint", 1, 4, 4, 3); __PYX_ERR(0, 406, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "findint") < 0)) __PYX_ERR(0, 406, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_nums = values[0];
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 406, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[2]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 406, __pyx_L3_error)
    __pyx_v_comp_ne = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_comp_ne == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 407, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("findint", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 406, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.findint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_38findint(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_nums, __pyx_v_begin, __pyx_v_end, __pyx_v_comp_ne);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_38findint(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_nums, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_v_comp_ne) {
  unsigned long *__pyx_v_longs;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  long __pyx_v_i;
  long __pyx_v_q;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  unsigned long __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  char const *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("findint", 0);

  /* "_gappedbuf.pyx":411
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_1 = (0 <= __pyx_v_begin);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "_gappedbuf.pyx":413
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         size = len(nums)
 */
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":412
 * 
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 412, __pyx_L1_error)

    /* "_gappedbuf.pyx":411
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":415
 *                                 begin, end, self.numelems))
 * 
 *         size = len(nums)             # <<<<<<<<<<<<<<
 *         longs = <unsigned long *>mem.PyMem_Malloc(
 *                                     size * sizeof(unsigned long))
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_nums); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 415, __pyx_L1_error)
  __pyx_v_size = __pyx_t_7;

  /* "_gappedbuf.pyx":416
 * 
 *         size = len(nums)
 *         longs = <unsigned long *>mem.PyMem_Malloc(             # <<<<<<<<<<<<<<
 *                                     size * sizeof(unsigned long))
 *         if not longs:
 */
  __pyx_v_longs = ((unsigned long *)PyMem_Malloc((__pyx_v_size * (sizeof(unsigned long)))));

  /* "_gappedbuf.pyx":418
 *         longs = <unsigned long *>mem.PyMem_Malloc(
 *                                     size * sizeof(unsigned long))
 *         if not longs:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_longs != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "_gappedbuf.pyx":419
 *                                     size * sizeof(unsigned long))
 *         if not longs:
 *             raise MemoryError()             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
    PyErr_NoMemory(); __PYX_ERR(0, 419, __pyx_L1_error)

    /* "_gappedbuf.pyx":418
 *         longs = <unsigned long *>mem.PyMem_Malloc(
 *                                     size * sizeof(unsigned long))
 *         if not longs:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 * 
 */
  }

  /* "_gappedbuf.pyx":421
 *             raise MemoryError()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             for i from 0 <= i < size:
 *                 longs[i] = nums[i]
 */
  /*try:*/ {

    /* "_gappedbuf.pyx":422
 * 
 *         try:
 *             for i from 0 <= i < size:             # <<<<<<<<<<<<<<
 *                 longs[i] = nums[i]
 * 
 */
    __pyx_t_7 = __pyx_v_size;
    for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

      /* "_gappedbuf.pyx":423
 *         try:
 *             for i from 0 <= i < size:
 *                 longs[i] = nums[i]             # <<<<<<<<<<<<<<
 * 
 *             # search before gap
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_nums, __pyx_v_i, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 423, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = __Pyx_PyInt_As_unsigned_long(__pyx_t_6); if (unlikely((__pyx_t_8 == (unsigned long)-1) && PyErr_Occurred())) __PYX_ERR(0, 423, __pyx_L6_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      (__pyx_v_longs[__pyx_v_i]) = __pyx_t_8;
    }

    /* "_gappedbuf.pyx":426
 * 
 *             # search before gap
 *             e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *             for p from begin <= p < e:
 *                 if not comp_ne:
 */
    __pyx_t_7 = __pyx_v_self->gap;
    __pyx_t_9 = __pyx_v_end;
    if (((__pyx_t_7 < __pyx_t_9) != 0)) {
      __pyx_t_10 = __pyx_t_7;
    } else {
      __pyx_t_10 = __pyx_t_9;
    }
    __pyx_v_e = __pyx_t_10;

    /* "_gappedbuf.pyx":427
 *             # search before gap
 *             e = min(end, self.gap)
 *             for p from begin <= p < e:             # <<<<<<<<<<<<<<
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 */
    __pyx_t_10 = __pyx_v_e;
    for (__pyx_v_p = __pyx_v_begin; __pyx_v_p < __pyx_t_10; __pyx_v_p++) {

      /* "_gappedbuf.pyx":428
 *             e = min(end, self.gap)
 *             for p from begin <= p < e:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
      __pyx_t_2 = ((!(__pyx_v_comp_ne != 0)) != 0);
      if (__pyx_t_2) {

        /* "_gappedbuf.pyx":429
 *             for p from begin <= p < e:
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             return p
 */
        __pyx_t_7 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_7; __pyx_v_q++) {

          /* "_gappedbuf.pyx":430
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p
 *                 else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":431
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             return p             # <<<<<<<<<<<<<<
 *                 else:
 *                     for q from 0 <= q < size:
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_p); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 431, __pyx_L6_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_r = __pyx_t_6;
            __pyx_t_6 = 0;
            goto __pyx_L5_return;

            /* "_gappedbuf.pyx":430
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p
 *                 else:
 */
          }
        }

        /* "_gappedbuf.pyx":428
 *             e = min(end, self.gap)
 *             for p from begin <= p < e:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
        goto __pyx_L12;
      }

      /* "_gappedbuf.pyx":433
 *                             return p
 *                 else:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             break
 */
      /*else*/ {
        __pyx_t_7 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_7; __pyx_v_q++) {

          /* "_gappedbuf.pyx":434
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":435
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             break             # <<<<<<<<<<<<<<
 *                     else:
 *                         return p
 */
            goto __pyx_L17_break;

            /* "_gappedbuf.pyx":434
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          }
        }
        /*else*/ {

          /* "_gappedbuf.pyx":437
 *                             break
 *                     else:
 *                         return p             # <<<<<<<<<<<<<<
 * 
 *             # search after gap
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_p); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 437, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L5_return;
        }
        __pyx_L17_break:;
      }
      __pyx_L12:;
    }

    /* "_gappedbuf.pyx":440
 * 
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *             for p from b <= p < end+self.gapsize:
 *                 if not comp_ne:
 */
    __pyx_t_10 = __pyx_v_self->gap;
    __pyx_t_7 = __pyx_v_begin;
    if (((__pyx_t_10 > __pyx_t_7) != 0)) {
      __pyx_t_9 = __pyx_t_10;
    } else {
      __pyx_t_9 = __pyx_t_7;
    }
    __pyx_v_b = (__pyx_t_9 + __pyx_v_self->gapsize);

    /* "_gappedbuf.pyx":441
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from b <= p < end+self.gapsize:             # <<<<<<<<<<<<<<
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 */
    __pyx_t_9 = (__pyx_v_end + __pyx_v_self->gapsize);
    for (__pyx_v_p = __pyx_v_b; __pyx_v_p < __pyx_t_9; __pyx_v_p++) {

      /* "_gappedbuf.pyx":442
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from b <= p < end+self.gapsize:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
      __pyx_t_2 = ((!(__pyx_v_comp_ne != 0)) != 0);
      if (__pyx_t_2) {

        /* "_gappedbuf.pyx":443
 *             for p from b <= p < end+self.gapsize:
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             return p-self.gapsize
 */
        __pyx_t_10 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_10; __pyx_v_q++) {

          /* "_gappedbuf.pyx":444
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p-self.gapsize
 *                 else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":445
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             return p-self.gapsize             # <<<<<<<<<<<<<<
 *                 else:
 *                     for q from 0 <= q < size:
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_p - __pyx_v_self->gapsize)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 445, __pyx_L6_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_r = __pyx_t_6;
            __pyx_t_6 = 0;
            goto __pyx_L5_return;

            /* "_gappedbuf.pyx":444
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p-self.gapsize
 *                 else:
 */
          }
        }

        /* "_gappedbuf.pyx":442
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from b <= p < end+self.gapsize:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
        goto __pyx_L21;
      }

      /* "_gappedbuf.pyx":447
 *                             return p-self.gapsize
 *                 else:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             break
 */
      /*else*/ {
        __pyx_t_10 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_10; __pyx_v_q++) {

          /* "_gappedbuf.pyx":448
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":449
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             break             # <<<<<<<<<<<<<<
 *                     else:
 *                         return p-self.gapsize
 */
            goto __pyx_L26_break;

            /* "_gappedbuf.pyx":448
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          }
        }
        /*else*/ {

          /* "_gappedbuf.pyx":451
 *                             break
 *                     else:
 *                         return p-self.gapsize             # <<<<<<<<<<<<<<
 *         finally:
 *             mem.PyMem_Free(longs)
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_p - __pyx_v_self->gapsize)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 451, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L5_return;
        }
        __pyx_L26_break:;
      }
      __pyx_L21:;
    }
  }

  /* "_gappedbuf.pyx":453
 *                         return p-self.gapsize
 *         finally:
 *             mem.PyMem_Free(longs)             # <<<<<<<<<<<<<<
 * 
 *         return -1
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_longs);
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __pyx_t_11 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_13 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_longs);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
      }
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_ErrRestore(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_13;
      goto __pyx_L1_error;
    }
    __pyx_L5_return: {
      __pyx_t_19 = __pyx_r;
      __pyx_r = 0;
      PyMem_Free(__pyx_v_longs);
      __pyx_r = __pyx_t_19;
      __pyx_t_19 = 0;
      goto __pyx_L0;
    }
    __pyx_L7:;
  }

  /* "_gappedbuf.pyx":455
 *             mem.PyMem_Free(longs)
 * 
 *         return -1             # <<<<<<<<<<<<<<
 * 
 *     def rfindint(self, object nums, Py_ssize_t begin, Py_ssize_t end,
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_neg_1);
  __pyx_r = __pyx_int_neg_1;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":406
 *             self.buf[p] = v
 * 
 *     def findint(self, object nums, Py_ssize_t begin, Py_ssize_t end,             # <<<<<<<<<<<<<<
 *                 int comp_ne):
 *         cdef unsigned long *longs
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.findint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pyx":457
 *         return -1
 * 
 *     def rfindint(self, object nums, Py_ssize_t begin, Py_ssize_t end,             # <<<<<<<<<<<<<<
 *                  int comp_ne):
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_41rfindint(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_41rfindint(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_nums = 0;
  Py_ssize_t __pyx_v_begin;
  Py_ssize_t __pyx_v_end;
  int __pyx_v_comp_ne;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rfindint (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nums,&__pyx_n_s_begin,&__pyx_n_s_end,&__pyx_n_s_comp_ne,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nums)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_begin)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rfindint", 1, 4, 4, 1); __PYX_ERR(0, 457, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rfindint", 1, 4, 4, 2); __PYX_ERR(0, 457, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_comp_ne)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rfindint", 1, 4, 4, 3); __PYX_ERR(0, 457, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "rfindint") < 0)) __PYX_ERR(0, 457, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_nums = values[0];
    __pyx_v_begin = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_begin == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 457, __pyx_L3_error)
    __pyx_v_end = __Pyx_PyIndex_AsSsize_t(values[2]); if (unlikely((__pyx_v_end == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 457, __pyx_L3_error)
    __pyx_v_comp_ne = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_comp_ne == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 458, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("rfindint", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 457, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.rfindint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_40rfindint(((struct GappedBufferObj *)__pyx_v_self), __pyx_v_nums, __pyx_v_begin, __pyx_v_end, __pyx_v_comp_ne);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_40rfindint(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_nums, Py_ssize_t __pyx_v_begin, Py_ssize_t __pyx_v_end, int __pyx_v_comp_ne) {
  unsigned long *__pyx_v_longs;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_v_b;
  Py_ssize_t __pyx_v_e;
  Py_ssize_t __pyx_v_p;
  long __pyx_v_i;
  long __pyx_v_q;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  unsigned long __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  char const *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("rfindint", 0);

  /* "_gappedbuf.pyx":463
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  __pyx_t_1 = (0 <= __pyx_v_begin);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_begin <= __pyx_v_end);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_end <= __pyx_v_self->numelems);
    }
  }
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "_gappedbuf.pyx":465
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))             # <<<<<<<<<<<<<<
 * 
 *         size = len(nums)
 */
    __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_begin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;

    /* "_gappedbuf.pyx":464
 * 
 *         if not (0 <= begin <= end <= self.numelems):
 *             raise ValueError('Invalid range: %d, %d, %d' % (             # <<<<<<<<<<<<<<
 *                                 begin, end, self.numelems))
 * 
 */
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_range_d_d_d, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 464, __pyx_L1_error)

    /* "_gappedbuf.pyx":463
 *         cdef Py_ssize_t size, b, e, p
 * 
 *         if not (0 <= begin <= end <= self.numelems):             # <<<<<<<<<<<<<<
 *             raise ValueError('Invalid range: %d, %d, %d' % (
 *                                 begin, end, self.numelems))
 */
  }

  /* "_gappedbuf.pyx":467
 *                                 begin, end, self.numelems))
 * 
 *         size = len(nums)             # <<<<<<<<<<<<<<
 *         longs = <unsigned long *>mem.PyMem_Malloc(
 *                                     size * sizeof(unsigned long))
 */
  __pyx_t_7 = PyObject_Length(__pyx_v_nums); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 467, __pyx_L1_error)
  __pyx_v_size = __pyx_t_7;

  /* "_gappedbuf.pyx":468
 * 
 *         size = len(nums)
 *         longs = <unsigned long *>mem.PyMem_Malloc(             # <<<<<<<<<<<<<<
 *                                     size * sizeof(unsigned long))
 *         if not longs:
 */
  __pyx_v_longs = ((unsigned long *)PyMem_Malloc((__pyx_v_size * (sizeof(unsigned long)))));

  /* "_gappedbuf.pyx":470
 *         longs = <unsigned long *>mem.PyMem_Malloc(
 *                                     size * sizeof(unsigned long))
 *         if not longs:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 *         try:
 */
  __pyx_t_2 = ((!(__pyx_v_longs != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "_gappedbuf.pyx":471
 *                                     size * sizeof(unsigned long))
 *         if not longs:
 *             raise MemoryError()             # <<<<<<<<<<<<<<
 *         try:
 *             for i from 0 <= i < size:
 */
    PyErr_NoMemory(); __PYX_ERR(0, 471, __pyx_L1_error)

    /* "_gappedbuf.pyx":470
 *         longs = <unsigned long *>mem.PyMem_Malloc(
 *                                     size * sizeof(unsigned long))
 *         if not longs:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 *         try:
 */
  }

  /* "_gappedbuf.pyx":472
 *         if not longs:
 *             raise MemoryError()
 *         try:             # <<<<<<<<<<<<<<
 *             for i from 0 <= i < size:
 *                 longs[i] = nums[i]
 */
  /*try:*/ {

    /* "_gappedbuf.pyx":473
 *             raise MemoryError()
 *         try:
 *             for i from 0 <= i < size:             # <<<<<<<<<<<<<<
 *                 longs[i] = nums[i]
 * 
 */
    __pyx_t_7 = __pyx_v_size;
    for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

      /* "_gappedbuf.pyx":474
 *         try:
 *             for i from 0 <= i < size:
 *                 longs[i] = nums[i]             # <<<<<<<<<<<<<<
 * 
 *             # search after gap
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_nums, __pyx_v_i, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 474, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = __Pyx_PyInt_As_unsigned_long(__pyx_t_6); if (unlikely((__pyx_t_8 == (unsigned long)-1) && PyErr_Occurred())) __PYX_ERR(0, 474, __pyx_L6_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      (__pyx_v_longs[__pyx_v_i]) = __pyx_t_8;
    }

    /* "_gappedbuf.pyx":477
 * 
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize             # <<<<<<<<<<<<<<
 *             for p from (end+self.gapsize) > p >= b:
 *                 if not comp_ne:
 */
    __pyx_t_7 = __pyx_v_self->gap;
    __pyx_t_9 = __pyx_v_begin;
    if (((__pyx_t_7 > __pyx_t_9) != 0)) {
      __pyx_t_10 = __pyx_t_7;
    } else {
      __pyx_t_10 = __pyx_t_9;
    }
    __pyx_v_b = (__pyx_t_10 + __pyx_v_self->gapsize);

    /* "_gappedbuf.pyx":478
 *             # search after gap
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from (end+self.gapsize) > p >= b:             # <<<<<<<<<<<<<<
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 */
    __pyx_t_10 = __pyx_v_b;
    for (__pyx_v_p = (__pyx_v_end + __pyx_v_self->gapsize)-1; __pyx_v_p >= __pyx_t_10; __pyx_v_p--) {

      /* "_gappedbuf.pyx":479
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from (end+self.gapsize) > p >= b:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
      __pyx_t_2 = ((!(__pyx_v_comp_ne != 0)) != 0);
      if (__pyx_t_2) {

        /* "_gappedbuf.pyx":480
 *             for p from (end+self.gapsize) > p >= b:
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             return p-self.gapsize
 */
        __pyx_t_7 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_7; __pyx_v_q++) {

          /* "_gappedbuf.pyx":481
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p-self.gapsize
 *                 else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":482
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             return p-self.gapsize             # <<<<<<<<<<<<<<
 *                 else:
 *                     for q from 0 <= q < size:
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_p - __pyx_v_self->gapsize)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 482, __pyx_L6_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_r = __pyx_t_6;
            __pyx_t_6 = 0;
            goto __pyx_L5_return;

            /* "_gappedbuf.pyx":481
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p-self.gapsize
 *                 else:
 */
          }
        }

        /* "_gappedbuf.pyx":479
 *             b = max(begin, self.gap) + self.gapsize
 *             for p from (end+self.gapsize) > p >= b:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
        goto __pyx_L12;
      }

      /* "_gappedbuf.pyx":484
 *                             return p-self.gapsize
 *                 else:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             break
 */
      /*else*/ {
        __pyx_t_7 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_7; __pyx_v_q++) {

          /* "_gappedbuf.pyx":485
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":486
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             break             # <<<<<<<<<<<<<<
 *                     else:
 *                         return p-self.gapsize
 */
            goto __pyx_L17_break;

            /* "_gappedbuf.pyx":485
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          }
        }
        /*else*/ {

          /* "_gappedbuf.pyx":488
 *                             break
 *                     else:
 *                         return p-self.gapsize             # <<<<<<<<<<<<<<
 * 
 *             # search before gap
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_p - __pyx_v_self->gapsize)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 488, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L5_return;
        }
        __pyx_L17_break:;
      }
      __pyx_L12:;
    }

    /* "_gappedbuf.pyx":491
 * 
 *             # search before gap
 *             e = min(end, self.gap)             # <<<<<<<<<<<<<<
 *             for p from e > p >= begin:
 *                 if not comp_ne:
 */
    __pyx_t_10 = __pyx_v_self->gap;
    __pyx_t_7 = __pyx_v_end;
    if (((__pyx_t_10 < __pyx_t_7) != 0)) {
      __pyx_t_9 = __pyx_t_10;
    } else {
      __pyx_t_9 = __pyx_t_7;
    }
    __pyx_v_e = __pyx_t_9;

    /* "_gappedbuf.pyx":492
 *             # search before gap
 *             e = min(end, self.gap)
 *             for p from e > p >= begin:             # <<<<<<<<<<<<<<
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 */
    __pyx_t_9 = __pyx_v_begin;
    for (__pyx_v_p = __pyx_v_e-1; __pyx_v_p >= __pyx_t_9; __pyx_v_p--) {

      /* "_gappedbuf.pyx":493
 *             e = min(end, self.gap)
 *             for p from e > p >= begin:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
      __pyx_t_2 = ((!(__pyx_v_comp_ne != 0)) != 0);
      if (__pyx_t_2) {

        /* "_gappedbuf.pyx":494
 *             for p from e > p >= begin:
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             return p
 */
        __pyx_t_10 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_10; __pyx_v_q++) {

          /* "_gappedbuf.pyx":495
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p
 *                 else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":496
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             return p             # <<<<<<<<<<<<<<
 *                 else:
 *                     for q from 0 <= q < size:
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_p); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 496, __pyx_L6_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_r = __pyx_t_6;
            __pyx_t_6 = 0;
            goto __pyx_L5_return;

            /* "_gappedbuf.pyx":495
 *                 if not comp_ne:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             return p
 *                 else:
 */
          }
        }

        /* "_gappedbuf.pyx":493
 *             e = min(end, self.gap)
 *             for p from e > p >= begin:
 *                 if not comp_ne:             # <<<<<<<<<<<<<<
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 */
        goto __pyx_L21;
      }

      /* "_gappedbuf.pyx":498
 *                             return p
 *                 else:
 *                     for q from 0 <= q < size:             # <<<<<<<<<<<<<<
 *                         if self.buf[p] == longs[q]:
 *                             break
 */
      /*else*/ {
        __pyx_t_10 = __pyx_v_size;
        for (__pyx_v_q = 0; __pyx_v_q < __pyx_t_10; __pyx_v_q++) {

          /* "_gappedbuf.pyx":499
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          __pyx_t_2 = (((__pyx_v_self->buf[__pyx_v_p]) == (__pyx_v_longs[__pyx_v_q])) != 0);
          if (__pyx_t_2) {

            /* "_gappedbuf.pyx":500
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:
 *                             break             # <<<<<<<<<<<<<<
 *                     else:
 *                         return p
 */
            goto __pyx_L26_break;

            /* "_gappedbuf.pyx":499
 *                 else:
 *                     for q from 0 <= q < size:
 *                         if self.buf[p] == longs[q]:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          }
        }
        /*else*/ {

          /* "_gappedbuf.pyx":502
 *                             break
 *                     else:
 *                         return p             # <<<<<<<<<<<<<<
 *         finally:
 *             mem.PyMem_Free(longs)
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_p); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 502, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_r = __pyx_t_6;
          __pyx_t_6 = 0;
          goto __pyx_L5_return;
        }
        __pyx_L26_break:;
      }
      __pyx_L21:;
    }
  }

  /* "_gappedbuf.pyx":504
 *                         return p
 *         finally:
 *             mem.PyMem_Free(longs)             # <<<<<<<<<<<<<<
 * 
 *         return -1
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_longs);
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __pyx_t_11 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_13 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_longs);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
      }
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_ErrRestore(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
      __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_13;
      goto __pyx_L1_error;
    }
    __pyx_L5_return: {
      __pyx_t_19 = __pyx_r;
      __pyx_r = 0;
      PyMem_Free(__pyx_v_longs);
      __pyx_r = __pyx_t_19;
      __pyx_t_19 = 0;
      goto __pyx_L0;
    }
    __pyx_L7:;
  }

  /* "_gappedbuf.pyx":506
 *             mem.PyMem_Free(longs)
 * 
 *         return -1             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_int_neg_1);
  __pyx_r = __pyx_int_neg_1;
  goto __pyx_L0;

  /* "_gappedbuf.pyx":457
 *         return -1
 * 
 *     def rfindint(self, object nums, Py_ssize_t begin, Py_ssize_t end,             # <<<<<<<<<<<<<<
 *                  int comp_ne):
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.rfindint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pxd":3
 * 
 * cdef public class GappedBuffer [object GappedBufferObj, type GappedBufferType]:
 *     cdef public Py_ssize_t bufsize             # <<<<<<<<<<<<<<
 *     cdef public Py_ssize_t numelems
 *     cdef public Py_ssize_t gap
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_7bufsize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_7bufsize_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_7bufsize___get__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_7bufsize___get__(struct GappedBufferObj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->bufsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.bufsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10_gappedbuf_12GappedBuffer_7bufsize_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10_gappedbuf_12GappedBuffer_7bufsize_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_7bufsize_2__set__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10_gappedbuf_12GappedBuffer_7bufsize_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyIndex_AsSsize_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 3, __pyx_L1_error)
  __pyx_v_self->bufsize = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.bufsize.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pxd":4
 * cdef public class GappedBuffer [object GappedBufferObj, type GappedBufferType]:
 *     cdef public Py_ssize_t bufsize
 *     cdef public Py_ssize_t numelems             # <<<<<<<<<<<<<<
 *     cdef public Py_ssize_t gap
 *     cdef public Py_ssize_t gapsize
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_8numelems_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_8numelems_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_8numelems___get__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_8numelems___get__(struct GappedBufferObj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->numelems); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.numelems.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10_gappedbuf_12GappedBuffer_8numelems_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10_gappedbuf_12GappedBuffer_8numelems_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_8numelems_2__set__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10_gappedbuf_12GappedBuffer_8numelems_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyIndex_AsSsize_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 4, __pyx_L1_error)
  __pyx_v_self->numelems = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.numelems.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pxd":5
 *     cdef public Py_ssize_t bufsize
 *     cdef public Py_ssize_t numelems
 *     cdef public Py_ssize_t gap             # <<<<<<<<<<<<<<
 *     cdef public Py_ssize_t gapsize
 *     cdef Py_UCS4 *buf
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_3gap_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_3gap_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_3gap___get__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_3gap___get__(struct GappedBufferObj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->gap); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.gap.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10_gappedbuf_12GappedBuffer_3gap_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10_gappedbuf_12GappedBuffer_3gap_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_3gap_2__set__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10_gappedbuf_12GappedBuffer_3gap_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyIndex_AsSsize_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 5, __pyx_L1_error)
  __pyx_v_self->gap = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.gap.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gappedbuf.pxd":6
 *     cdef public Py_ssize_t numelems
 *     cdef public Py_ssize_t gap
 *     cdef public Py_ssize_t gapsize             # <<<<<<<<<<<<<<
 *     cdef Py_UCS4 *buf
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_7gapsize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_7gapsize_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_7gapsize___get__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_7gapsize___get__(struct GappedBufferObj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->gapsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.gapsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_10_gappedbuf_12GappedBuffer_7gapsize_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_10_gappedbuf_12GappedBuffer_7gapsize_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_7gapsize_2__set__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_10_gappedbuf_12GappedBuffer_7gapsize_2__set__(struct GappedBufferObj *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyIndex_AsSsize_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 6, __pyx_L1_error)
  __pyx_v_self->gapsize = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.gapsize.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_43__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_43__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_42__reduce_cython__(((struct GappedBufferObj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_42__reduce_cython__(CYTHON_UNUSED struct GappedBufferObj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_45__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_10_gappedbuf_12GappedBuffer_45__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_10_gappedbuf_12GappedBuffer_44__setstate_cython__(((struct GappedBufferObj *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10_gappedbuf_12GappedBuffer_44__setstate_cython__(CYTHON_UNUSED struct GappedBufferObj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gappedbuf.GappedBuffer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_10_gappedbuf_GappedBuffer __pyx_vtable_10_gappedbuf_GappedBuffer;

static PyObject *__pyx_tp_new_10_gappedbuf_GappedBuffer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct GappedBufferObj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct GappedBufferObj *)o);
  p->__pyx_vtab = __pyx_vtabptr_10_gappedbuf_GappedBuffer;
  if (unlikely(__pyx_pw_10_gappedbuf_12GappedBuffer_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_10_gappedbuf_GappedBuffer(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_10_gappedbuf_12GappedBuffer_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}
static PyObject *__pyx_sq_item_10_gappedbuf_GappedBuffer(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_10_gappedbuf_GappedBuffer(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_10_gappedbuf_12GappedBuffer_19__setitem__(o, i, v);
  }
  else {
    return __pyx_pw_10_gappedbuf_12GappedBuffer_21__delitem__(o, i);
  }
}

static PyObject *__pyx_getprop_10_gappedbuf_12GappedBuffer_bufsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10_gappedbuf_12GappedBuffer_7bufsize_1__get__(o);
}

static int __pyx_setprop_10_gappedbuf_12GappedBuffer_bufsize(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10_gappedbuf_12GappedBuffer_7bufsize_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10_gappedbuf_12GappedBuffer_numelems(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10_gappedbuf_12GappedBuffer_8numelems_1__get__(o);
}

static int __pyx_setprop_10_gappedbuf_12GappedBuffer_numelems(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10_gappedbuf_12GappedBuffer_8numelems_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10_gappedbuf_12GappedBuffer_gap(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10_gappedbuf_12GappedBuffer_3gap_1__get__(o);
}

static int __pyx_setprop_10_gappedbuf_12GappedBuffer_gap(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10_gappedbuf_12GappedBuffer_3gap_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_10_gappedbuf_12GappedBuffer_gapsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_10_gappedbuf_12GappedBuffer_7gapsize_1__get__(o);
}

static int __pyx_setprop_10_gappedbuf_12GappedBuffer_gapsize(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_10_gappedbuf_12GappedBuffer_7gapsize_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_10_gappedbuf_GappedBuffer[] = {
  {"insert", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_5insert, METH_VARARGS|METH_KEYWORDS, 0},
  {"append", (PyCFunction)__pyx_pw_10_gappedbuf_12GappedBuffer_7append, METH_O, 0},
  {"delete", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_9delete, METH_VARARGS|METH_KEYWORDS, 0},
  {"replace", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_11replace, METH_VARARGS|METH_KEYWORDS, 0},
  {"get", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_13get, METH_VARARGS|METH_KEYWORDS, 0},
  {"findchr", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_23findchr, METH_VARARGS|METH_KEYWORDS, 0},
  {"rfindchr", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_25rfindchr, METH_VARARGS|METH_KEYWORDS, 0},
  {"getint", (PyCFunction)__pyx_pw_10_gappedbuf_12GappedBuffer_27getint, METH_O, 0},
  {"getints", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_29getints, METH_VARARGS|METH_KEYWORDS, 0},
  {"insertints", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_31insertints, METH_VARARGS|METH_KEYWORDS, 0},
  {"appendints", (PyCFunction)__pyx_pw_10_gappedbuf_12GappedBuffer_33appendints, METH_O, 0},
  {"replaceints", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_35replaceints, METH_VARARGS|METH_KEYWORDS, 0},
  {"setints", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_37setints, METH_VARARGS|METH_KEYWORDS, 0},
  {"findint", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_39findint, METH_VARARGS|METH_KEYWORDS, 0},
  {"rfindint", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_10_gappedbuf_12GappedBuffer_41rfindint, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_10_gappedbuf_12GappedBuffer_43__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_10_gappedbuf_12GappedBuffer_45__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_10_gappedbuf_GappedBuffer[] = {
  {(char *)"bufsize", __pyx_getprop_10_gappedbuf_12GappedBuffer_bufsize, __pyx_setprop_10_gappedbuf_12GappedBuffer_bufsize, (char *)0, 0},
  {(char *)"numelems", __pyx_getprop_10_gappedbuf_12GappedBuffer_numelems, __pyx_setprop_10_gappedbuf_12GappedBuffer_numelems, (char *)0, 0},
  {(char *)"gap", __pyx_getprop_10_gappedbuf_12GappedBuffer_gap, __pyx_setprop_10_gappedbuf_12GappedBuffer_gap, (char *)0, 0},
  {(char *)"gapsize", __pyx_getprop_10_gappedbuf_12GappedBuffer_gapsize, __pyx_setprop_10_gappedbuf_12GappedBuffer_gapsize, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_GappedBuffer = {
  __pyx_pw_10_gappedbuf_12GappedBuffer_15__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_10_gappedbuf_GappedBuffer, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_GappedBuffer = {
  __pyx_pw_10_gappedbuf_12GappedBuffer_15__len__, /*mp_length*/
  __pyx_pw_10_gappedbuf_12GappedBuffer_17__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_10_gappedbuf_GappedBuffer, /*mp_ass_subscript*/
};

DL_EXPORT(PyTypeObject) GappedBufferType = {
  PyVarObject_HEAD_INIT(0, 0)
  "_gappedbuf.GappedBuffer", /*tp_name*/
  sizeof(struct GappedBufferObj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_10_gappedbuf_GappedBuffer, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_GappedBuffer, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_GappedBuffer, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_10_gappedbuf_GappedBuffer, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_10_gappedbuf_GappedBuffer, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_10_gappedbuf_GappedBuffer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__gappedbuf(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__gappedbuf},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_gappedbuf",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_GappedBuffer, __pyx_k_GappedBuffer, sizeof(__pyx_k_GappedBuffer), 0, 0, 1, 1},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_kp_s_Invalid_index_value_d_d, __pyx_k_Invalid_index_value_d_d, sizeof(__pyx_k_Invalid_index_value_d_d), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_range_d_d_d, __pyx_k_Invalid_range_d_d_d, sizeof(__pyx_k_Invalid_range_d_d_d), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_string_length_d, __pyx_k_Invalid_string_length_d, sizeof(__pyx_k_Invalid_string_length_d), 0, 0, 1, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_appendints, __pyx_k_appendints, sizeof(__pyx_k_appendints), 0, 0, 1, 1},
  {&__pyx_n_s_begin, __pyx_k_begin, sizeof(__pyx_k_begin), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_comp_ne, __pyx_k_comp_ne, sizeof(__pyx_k_comp_ne), 0, 0, 1, 1},
  {&__pyx_n_s_delete, __pyx_k_delete, sizeof(__pyx_k_delete), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_gappedbuf, __pyx_k_gappedbuf, sizeof(__pyx_k_gappedbuf), 0, 0, 1, 1},
  {&__pyx_kp_s_gappedbuf__gappedbuf_pyx, __pyx_k_gappedbuf__gappedbuf_pyx, sizeof(__pyx_k_gappedbuf__gappedbuf_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_gappedbufre, __pyx_k_gappedbufre, sizeof(__pyx_k_gappedbufre), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getint, __pyx_k_getint, sizeof(__pyx_k_getint), 0, 0, 1, 1},
  {&__pyx_n_s_getints, __pyx_k_getints, sizeof(__pyx_k_getints), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_init_gappedbufre, __pyx_k_init_gappedbufre, sizeof(__pyx_k_init_gappedbufre), 0, 0, 1, 1},
  {&__pyx_n_s_insert, __pyx_k_insert, sizeof(__pyx_k_insert), 0, 0, 1, 1},
  {&__pyx_n_s_insertints, __pyx_k_insertints, sizeof(__pyx_k_insertints), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_modules, __pyx_k_modules, sizeof(__pyx_k_modules), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_nums, __pyx_k_nums, sizeof(__pyx_k_nums), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_replace, __pyx_k_replace, sizeof(__pyx_k_replace), 0, 0, 1, 1},
  {&__pyx_n_s_replaceints, __pyx_k_replaceints, sizeof(__pyx_k_replaceints), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_setints, __pyx_k_setints, sizeof(__pyx_k_setints), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_kp_s_step_is_not_supported, __pyx_k_step_is_not_supported, sizeof(__pyx_k_step_is_not_supported), 0, 0, 1, 0},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_v, __pyx_k_v, sizeof(__pyx_k_v), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 24, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 84, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 191, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(0, 207, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "_gappedbuf.pyx":222
 *                      &step, &slicelength)
 *             if step != 1:
 *                 raise ValueError('step is not supported')             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_step_is_not_supported); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "_gappedbuf.pyx":14
 * 
 * 
 * def init_gappedbufre():             # <<<<<<<<<<<<<<
 *     import sys
 *     sys.modules['_gappedbufre'] = PyInit__gappedbufre()
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_n_s_sys); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_gappedbuf__gappedbuf_pyx, __pyx_n_s_init_gappedbufre, 14, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_10_gappedbuf_GappedBuffer = &__pyx_vtable_10_gappedbuf_GappedBuffer;
  __pyx_vtable_10_gappedbuf_GappedBuffer._resize_buf = (void (*)(struct GappedBufferObj *, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__resize_buf;
  __pyx_vtable_10_gappedbuf_GappedBuffer._expand_gap = (void (*)(struct GappedBufferObj *, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__expand_gap;
  __pyx_vtable_10_gappedbuf_GappedBuffer._shrink_gap = (void (*)(struct GappedBufferObj *))__pyx_f_10_gappedbuf_12GappedBuffer__shrink_gap;
  __pyx_vtable_10_gappedbuf_GappedBuffer._move_gap = (void (*)(struct GappedBufferObj *, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__move_gap;
  __pyx_vtable_10_gappedbuf_GappedBuffer._get_max = (Py_UCS4 (*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__get_max;
  __pyx_vtable_10_gappedbuf_GappedBuffer._copy_chars1 = (void (*)(struct GappedBufferObj *, void *, Py_ssize_t, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__copy_chars1;
  __pyx_vtable_10_gappedbuf_GappedBuffer._copy_chars2 = (void (*)(struct GappedBufferObj *, void *, Py_ssize_t, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__copy_chars2;
  __pyx_vtable_10_gappedbuf_GappedBuffer._copy_chars4 = (void (*)(struct GappedBufferObj *, void *, Py_ssize_t, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__copy_chars4;
  __pyx_vtable_10_gappedbuf_GappedBuffer._get_slice = (int (*)(struct GappedBufferObj *, PyObject *, Py_ssize_t *, Py_ssize_t *))__pyx_f_10_gappedbuf_12GappedBuffer__get_slice;
  __pyx_vtable_10_gappedbuf_GappedBuffer._insert = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, PyObject *))__pyx_f_10_gappedbuf_12GappedBuffer__insert;
  __pyx_vtable_10_gappedbuf_GappedBuffer.insert = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, PyObject *, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_insert;
  __pyx_vtable_10_gappedbuf_GappedBuffer.append = (PyObject *(*)(struct GappedBufferObj *, PyObject *, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_append;
  __pyx_vtable_10_gappedbuf_GappedBuffer._delete = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t))__pyx_f_10_gappedbuf_12GappedBuffer__delete;
  __pyx_vtable_10_gappedbuf_GappedBuffer.delete = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_delete;
  __pyx_vtable_10_gappedbuf_GappedBuffer.replace = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, PyObject *, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_replace;
  __pyx_vtable_10_gappedbuf_GappedBuffer.get = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_get;
  __pyx_vtable_10_gappedbuf_GappedBuffer.getint = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_getint;
  __pyx_vtable_10_gappedbuf_GappedBuffer.getints = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_getints;
  __pyx_vtable_10_gappedbuf_GappedBuffer._insertints = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, PyObject *))__pyx_f_10_gappedbuf_12GappedBuffer__insertints;
  __pyx_vtable_10_gappedbuf_GappedBuffer.insertints = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, PyObject *, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_insertints;
  __pyx_vtable_10_gappedbuf_GappedBuffer.appendints = (PyObject *(*)(struct GappedBufferObj *, PyObject *, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_appendints;
  __pyx_vtable_10_gappedbuf_GappedBuffer.replaceints = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, PyObject *, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_replaceints;
  __pyx_vtable_10_gappedbuf_GappedBuffer.setints = (PyObject *(*)(struct GappedBufferObj *, Py_ssize_t, Py_ssize_t, unsigned long, int __pyx_skip_dispatch))__pyx_f_10_gappedbuf_12GappedBuffer_setints;
  if (PyType_Ready(&GappedBufferType) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  GappedBufferType.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!GappedBufferType.tp_dictoffset && GappedBufferType.tp_getattro == PyObject_GenericGetAttr)) {
    GappedBufferType.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(GappedBufferType.tp_dict, __pyx_vtabptr_10_gappedbuf_GappedBuffer) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_GappedBuffer, (PyObject *)&GappedBufferType) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&GappedBufferType) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __pyx_ptype_10_gappedbuf_GappedBuffer = &GappedBufferType;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(5, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_gappedbuf(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_gappedbuf(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__gappedbuf(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__gappedbuf(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__gappedbuf(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_gappedbuf' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__gappedbuf(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_gappedbuf", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main__gappedbuf) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "_gappedbuf")) {
      if (unlikely(PyDict_SetItemString(modules, "_gappedbuf", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "_gappedbuf.pyx":14
 * 
 * 
 * def init_gappedbufre():             # <<<<<<<<<<<<<<
 *     import sys
 *     sys.modules['_gappedbufre'] = PyInit__gappedbufre()
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_10_gappedbuf_1init_gappedbufre, NULL, __pyx_n_s_gappedbuf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_init_gappedbufre, __pyx_t_1) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gappedbuf.pyx":1
 * from libc.stdlib cimport malloc, free, realloc             # <<<<<<<<<<<<<<
 * from libc.string cimport memmove, memcpy
 * from cpython cimport unicode, mem, ref
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init _gappedbuf", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init _gappedbuf");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
    else state = (PyGILState_STATE)0;
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#if defined(__Pyx_CyFunction_USED) && defined(NDEBUG)
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* UnicodeAsUCS4 */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* CIntFromPy */
static CYTHON_INLINE unsigned long __Pyx_PyInt_As_unsigned_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned long neg_one = (unsigned long) -1, const_zero = (unsigned long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned long, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) >= 2 * PyLong_SHIFT) {
                            return (unsigned long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) >= 3 * PyLong_SHIFT) {
                            return (unsigned long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) >= 4 * PyLong_SHIFT) {
                            return (unsigned long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned long) (((unsigned long)-1)*(((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned long) ((((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned long) (((unsigned long)-1)*(((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned long) ((((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned long) (((unsigned long)-1)*(((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned long) ((((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned long) -1;
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned long) -1;
        val = __Pyx_PyInt_As_unsigned_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned long");
    return (unsigned long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned long");
    return (unsigned long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_long(unsigned long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned long neg_one = (unsigned long) -1, const_zero = (unsigned long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* ObjectAsUCS4 */
static Py_UCS4 __Pyx__PyObject_AsPy_UCS4_raise_error(long ival) {
   if (ival < 0) {
       if (!PyErr_Occurred())
           PyErr_SetString(PyExc_OverflowError,
                           "cannot convert negative value to Py_UCS4");
   } else {
       PyErr_SetString(PyExc_OverflowError,
                       "value too large to convert to Py_UCS4");
   }
   return (Py_UCS4)-1;
}
static Py_UCS4 __Pyx__PyObject_AsPy_UCS4(PyObject* x) {
   long ival;
   ival = __Pyx_PyInt_As_long(x);
   if (unlikely(!__Pyx_is_valid_index(ival, 1114111 + 1))) {
       return __Pyx__PyObject_AsPy_UCS4_raise_error(ival);
   }
   return (Py_UCS4)ival;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
